function [data] = parseControllerLog(filename)

fid = fopen(filename);


hipStruct = struct('position',[],'rate',[],'order',[]);

hipLegs = struct('left_fore',hipStruct,'right_fore',hipStruct,...
                 'left_hind',hipStruct,'right_hind',hipStruct);

kneeStruct = struct('position',[],'rate',[],...
                    'virtualPosition',[],'virtualRate',[],...
                    'kneeStiffness',[],'torque',[],'dampingTorque',[],...
                    'order',[]);

kneeLegs = struct('left_fore',kneeStruct,'right_fore',kneeStruct,...
                  'left_hind',kneeStruct,'right_hind',kneeStruct);

data = struct('time',[],'hip',hipLegs,'knee',kneeLegs);

while (1)
   tline = fgetl(fid);
   if (~ischar(tline))
      break; 
   end
   A = sscanf(tline,'pos : %e order : %e rate : %e ');
   data.hip.left_fore.position(end+1)=A(1);
   data.hip.left_fore.order(end+1)=A(2);
   data.hip.left_fore.rate(end+1)=A(3);
   tline = fgetl(fid);
   if (~ischar(tline))
      break; 
   end
   A = sscanf(tline,'pos : %e order : %e rate : %e ');
   data.hip.right_fore.position(end+1)=A(1);
   data.hip.right_fore.order(end+1)=A(2);
   data.hip.right_fore.rate(end+1)=A(3);
   tline = fgetl(fid);
   if (~ischar(tline))
      break; 
   end
   A = sscanf(tline,'pos : %e order : %e rate : %e ');
   data.hip.left_hind.position(end+1)=A(1);
   data.hip.left_hind.order(end+1)=A(2);
   data.hip.left_hind.rate(end+1)=A(3);
   tline = fgetl(fid);
   if (~ischar(tline))
      break; 
   end
   A = sscanf(tline,'pos : %e order : %e rate : %e ');
   data.hip.right_hind.position(end+1)=A(1);
   data.hip.right_hind.order(end+1)=A(2);
   data.hip.right_hind.rate(end+1)=A(3);
   
   tline = fgetl(fid);
   if (~ischar(tline))
      break; 
   end
   A = sscanf(tline,'x1 : %e x1D : %e order : %e x1Dt : %e x1DDt : %e kStiff : %e tau1 : %e dpTrq : %e rLen : %e');
   data.knee.left_fore.position(end+1) = A(1);
   data.knee.left_fore.virtualPosition(end+1) = A(2);
   data.knee.left_fore.order(end+1) = A(3);
   data.knee.left_fore.rate(end+1) = A(4);
   data.knee.left_fore.virtualRate(end+1) = A(5);
   data.knee.left_fore.kneeStiffness(end+1) = A(6);
   data.knee.left_fore.torque(end+1) = A(7);
   data.knee.left_fore.dampingTorque(end+1) = A(8);
   tline = fgetl(fid);
   if (~ischar(tline))
      break; 
   end
   A = sscanf(tline,'x1 : %e x1D : %e order : %e x1Dt : %e x1DDt : %e kStiff : %e tau1 : %e dpTrq : %e rLen : %e');
   data.knee.right_fore.position(end+1) = A(1);
   data.knee.right_fore.virtualPosition(end+1) = A(2);
   data.knee.right_fore.order(end+1) = A(3);
   data.knee.right_fore.rate(end+1) = A(4);
   data.knee.right_fore.virtualRate(end+1) = A(5);
   data.knee.right_fore.kneeStiffness(end+1)= A(6);
   data.knee.right_fore.torque(end+1) = A(7);
   data.knee.right_fore.dampingTorque(end+1) = A(8);
   tline = fgetl(fid);
   if (~ischar(tline))
      break; 
   end
   A = sscanf(tline,'x1 : %e x1D : %e order : %e x1Dt : %e x1DDt : %e kStiff : %e tau1 : %e dpTrq : %e rLen : %e');
   data.knee.left_hind.position(end+1) = A(1);
   data.knee.left_hind.virtualPosition(end+1) = A(2);
   data.knee.left_hind.order(end+1) = A(3);
   data.knee.left_hind.rate(end+1) = A(4);
   data.knee.left_hind.virtualRate(end+1) = A(5);
   data.knee.left_hind.kneeStiffness(end+1) = A(6);
   data.knee.left_hind.torque(end+1) = A(7);
   data.knee.left_hind.dampingTorque(end+1) = A(8);
   tline = fgetl(fid);
   if (~ischar(tline))
      break; 
   end
   A = sscanf(tline,'x1 : %e x1D : %e order : %e x1Dt : %e x1DDt : %e kStiff : %e tau1 : %e dpTrq : %e rLen : %e');
   data.knee.right_hind.position(end+1) = A(1);
   data.knee.right_hind.virtualPosition(end+1) = A(2);
   data.knee.right_hind.order(end+1) = A(3);
   data.knee.right_hind.rate(end+1) = A(4);
   data.knee.right_hind.virtualRate(end+1) = A(5);
   data.knee.right_hind.kneeStiffness(end+1) = A(6);
   data.knee.right_hind.torque(end+1) = A(7);
   data.knee.right_hind.dampingTorque(end+1) = A(8);
end
data.time=((0:1:(length(data.hip.left_fore.position)-1))*0.002);



end