# Cmake script ....

function(create_dir_if_not_exists DIRNAME)
    if(NOT EXISTS ${DIRNAME})
        file(MAKE_DIRECTORY ${DIRNAME})
    endif(NOT EXISTS ${DIRNAME})
endfunction(create_dir_if_not_exists)


function(my_copy_file FILEIN FILEOUT)
  execute_process(COMMAND "${CMAKE_COMMAND}" "-E" "copy" "${FILEIN}" "${FILEOUT}"
                  RESULT_VARIABLE RESULT
		  OUTPUT_VARIABLE COUT
		  ERROR_VARIABLE CERR)
#  message(STATUS "Copying ${FILEIN} ${FILEOUT}")
  if(RESULT)
    message(FATAL_ERROR "Failed to copy ${FILEIN} to ${FILEOUT}")
  endif(RESULT)
endfunction(my_copy_file)

if(NOT CREATE_WEBOTS_PROJECT_TARGET_INSTALL_DIR OR NOT CREATE_WEBOTS_PROJECT_TEMPLATE_DIR )
  if(NOT CREATE_WEBOTS_PROJECT_TARGET_INSTALL_DIR)
      set(REASON "You did not provide a target installation directory")
  endif(NOT CREATE_WEBOTS_PROJECT_TARGET_INSTALL_DIR)
  if(NOT CREATE_WEBOTS_PROJECT_TEMPLATE_DIR)
      set(REASON "You did not provide a template directory")
  endif(NOT CREATE_WEBOTS_PROJECT_TEMPLATE_DIR)
  
  message(FATAL_ERROR "${REASON}, so we abort")
  message(STATUS "Usage : cmake -P create-webots-project.cmake "
  "-DCREATE_WEBOTS_PROJECT_TARGET_INSTALL_DIR=path -DCREATE_WEBOTS_PROJECT_TEMPLATE_DIR=path"
  "[-DCREATE_WEBOTS_PROJECT_UPDATE=1] [-DCREATE_WEBOTS_PROJECT_FORCE=1]")

endif(NOT CREATE_WEBOTS_PROJECT_TARGET_INSTALL_DIR OR NOT CREATE_WEBOTS_PROJECT_TEMPLATE_DIR )

  
if(EXISTS ${CREATE_WEBOTS_PROJECT_TARGET_INSTALL_DIR}/controllers/basic/basic.cpp AND NOT CREATE_WEBOTS_PROJECT_UPDATE AND NOT CREATE_WEBOTS_PROJECT_FORCE )
  message(FATAL_ERROR "It seems that the target directory "
          "'${CREATE_WEBOTS_PROJECT_TARGET_INSTALL_DIR}' is already a "
          "libamarsi-quad project directory, and you did not use update or force" 
          "option.")
endif(EXISTS ${CREATE_WEBOTS_PROJECT_TARGET_INSTALL_DIR}/controllers/basic/basic.cpp AND NOT CREATE_WEBOTS_PROJECT_UPDATE AND NOT CREATE_WEBOTS_PROJECT_FORCE )
  
file(TO_CMAKE_PATH ${CREATE_WEBOTS_PROJECT_TARGET_INSTALL_DIR} MY_PREFIX )
create_dir_if_not_exists(${MY_PREFIX}/prefs )
create_dir_if_not_exists(${MY_PREFIX}/worlds )
create_dir_if_not_exists(${MY_PREFIX}/plugins )
create_dir_if_not_exists(${MY_PREFIX}/plugins/physics )
create_dir_if_not_exists(${MY_PREFIX}/plugins/physics/QuadAmarsi )
create_dir_if_not_exists(${MY_PREFIX}/controllers )
create_dir_if_not_exists(${MY_PREFIX}/controllers/basic )

file(GLOB_RECURSE FILE_TO_COPY RELATIVE ${CREATE_WEBOTS_PROJECT_TEMPLATE_DIR} ${CREATE_WEBOTS_PROJECT_TEMPLATE_DIR}/* )
    
foreach(fileto ${FILE_TO_COPY} )
  file(TO_NATIVE_PATH ${CREATE_WEBOTS_PROJECT_TEMPLATE_DIR}/${fileto} filein )
  file(TO_NATIVE_PATH ${MY_PREFIX}/${fileto} fileout )

  if(NOT CREATE_WEBOTS_PROJECT_UPDATE)

    message(STATUS "Installing file ${fileto} in ${MY_PREFIX}" )
    my_copy_file(${filein} ${fileout})

  else(NOT CREATE_WEBOTS_PROJECT_UPDATE)
    # we check if the file exists, 
    #and if it is different and older than the one we install
    #if it is the case trhen we make a backup and display a warning
    # if not then we can overwrite it.
    set(FILE_ARE_DIFFERENT 0)
    
    if(EXISTS ${fileout})
      # only test if fileout already exists
      #check if files are different
      execute_process(COMMAND "${CMAKE_COMMAND}" "-E" "compare_files" "${filein}" "${fileout}" 
                              RESULT_VARIABLE FILE_ARE_DIFFERENT 
			      OUTPUT_VARIABLE COUT
			      ERROR_VARIABLE CERR)
    endif(EXISTS ${fileout})
    if(FILE_ARE_DIFFERENT)
      if(filein IS_NEWER_THAN fileout) # here there is a conflict between user modification and new features, 
	#we copy fileout to fileout.time.bak and copy filein to fileout , and send message.
	#get timestamp

	execute_process(COMMAND "date" "+%s"
	                        OUTPUT_VARIABLE TIMESTAMP
				OUTPUT_STRIP_TRAILING_WHITESPACE)

	set(fileback "${fileout}.${TIMESTAMP}.bak")

	message(WARNING "It seems that you have a modified version of file ${fileout}. We move it to ${fileback}. You may have to change to merge your changes with the new file ${fileout}")

	my_copy_file(${fileout} ${fileback})

	my_copy_file(${filein} ${fileout})

      endif(filein IS_NEWER_THAN fileout)
      # there is some modification but no new features in the file, so it's okay we don't anything.
    else(FILE_ARE_DIFFERENT) # no difference, or no fileout, so we copy it
      my_copy_file(${filein} ${fileout})
      #message(STATUS "Installing file ${fileto} in ${MY_PREFIX}" )
    endif(FILE_ARE_DIFFERENT)
    
  endif(NOT CREATE_WEBOTS_PROJECT_UPDATE)
  
endforeach(fileto ${FILE_TO_COPY} )
    

