/**
 * \page model Model description
 *
 * The <a href="http://biorob.epfl.ch/page38266.html">Cheetah</a> robot features
 * three-segmented, compliant legs with “pantographic” [Witte 2003, "Quadrupedal
 * Mammals as Paragons for Walking Machines"] leg behavior : i.e. the first and
 * the third segment are oriented in parallel. The here provided, simplified
 * quadruped robot model features three leg segments of equal length. This
 * differs slightly from the publication [Ruthishauser et. al, 2008].
 *
 *
 * \section act Actuators characteristics
 *
 * \image html ModelActuator.png "The actuators of the model. There is only 2 per legs, the two knee joints acting conjointly."
 *
 * Currently there are 8 actuators in the robot model: two actuators per leg,
 * one for the hip, and one for the
 * knee.
 *
 * The proximal joint (hip) is responsible for leg retraction
 * (backwards) and protraction (swing forward). The distal actuated joint
 * (knee) is responsible for leg extension and leg flexion.
 *
 * \subsection hip Hip Actuators
 *
 * They are standard P-controlled servo motor with relatively high end motor
 * characteristics, as seen in the actuator characteristic table.
 *
 * \subsection knee Knee Actuator
 *
 * \image html ModelKnee.png "The knee actuator, that simulate the pantographic, compliant mechanism of the Cheetah."
 *
 * \subsection carac Sumarry of the carcteristic of the model, and interface with the API
 *
 * <table>
 *  <tr>
 *   <th>Name</th>
 *   <th>ActuatorID</th>
 *   <th>Orientation</th>
 *   <th> Position Range </th>
 *   <th>Max output torque</th>
 *   <th>Max output Velocity</th>
 *   <th>Control type </th>
 *  </tr>
 *  <tr>
 *   <td>Hip actuator</td>
 *   <td>
 *    \ref amarsi::LEFT_FORE_HIP
 *    \ref amarsi::RIGHT_FORE_HIP
 *    \ref amarsi::LEFT_HIND_HIP
 *    \ref amarsi::LEFT_HIND_HIP
 *   </td>
 *   <td>
 *      + : cranial rotation<br/>
 *      -- : caudal rotation
 *   </td>
 *   <td>
 *      <b>Forelimbs</b> : [-2.0 , 1.2 ]
 *      <b>Hindlimbs</b> : [-1.9 , 1.3 ]
 *   </td>
 *   <td>3 N.m </td>
 *   <td>29.60 rad.s^{-1} </td>
 *   <td>webots P-controlled</td>
 *  </tr>
 *  <tr>
 *   <td>Knee actuator</td>
 *   <td>
 *      \ref amarsi::LEFT_FORE_KNEE
 *      \ref amarsi::RIGHT_FORE_KNEE
 *      \ref amarsi::LEFT_HIND_KNEE
 *      \ref amarsi::LEFT_HIND_KNEE
 *   </td>
 *   <td>
 *    + : contraction<br/>
 *    -- : extension
 *    </td>
 *   <td>
 *    [0.0 , 1.98]
 *   </td>
 *   <td> 3 N.m for both servos </td>
 *   <td>
 *      <b>first</b> (compliant) servo : 10.0
 *
 *      <b>second</b> servo : 29.60
 *   </td>
 *   <td>
 *        <b>first</b> (compliant) servo : Perfect control,command is reached
 *         in one timestep if it is in servo limitation
 *
 *         <b>second</b> servo : webots P-controlled
 *   </td>
 *  </tr>
 * </table>
 * \section Sensors Sensors of the robot
 *
 * The model has several sensors :
 *  - Force sensor ( amarsi::Touch3DSensor )on the feet : They give the 3-axis
 *    force applied on the toe. They also give a filtered output that indicate
 *    if the toe touch the ground. This filtering is needed since high frequency
 *    (period of a few simulation step) oscillations of the legs could happen.
 *    The value returned are in the \f$ \left [ 0 , 100 \right]\;N.m\f$
 *    interval, and are given with a linear 12 bits precision in thsi interval.
 *  - Knee position encoder on the knee joint ( amarsi::PositionEncoder ). It
 *    returns the position of the first knee joint, relatively to the maximal
 *    extended position (0.0). The returned value is also given with a 12 bits
 *    precision.
 *  - A \ref Gyroscope and an \ref Accelerometer that gives the relative position from the
 *    start of the robot (start of the controller or just after a call to
 *    Supervisor::resetSimulation()). These values are integrated from the
 *    linear acceleration and angular speed, so they may drift over time.
 *
 *
 *  <table>
 *    <tr>
 *      <th>type</th>
 *      <th>location</th>
 *      <th>access method </th>
 *      <th>Interval of returned values</th>
 *      <th>returned value</th>
 *    </tr>
 *    <tr>
 *      <td>Absolute position encoder</td>
 *      <td>first (proximal) knee joints</td>
 *      <td>amarsi::Controller::kneeEncoder()</td>
 *      <td> [0,pi] rad.s^{-1}</td>
 *      <td> double with a simulated 12 bits precision </td>
 *    </tr>
 *    <tr>
 *      <td>Force sensor</td>
 *      <td>toes</td>
 *      <td>amarsi::Controller::touchSensor()</td>
 *      <td> [0,100] N.m</td>
 *      <td> double with a simulated 12 bits precision </td>
 *    </tr>
 *    <tr>
 *      <td>Gyroscope and Accelerometer</td>
 *      <td>toe</td>
 *      <td>amarsi::Controller::gyro() and amarsi::Controller::accelerometer()</td>
 *      <td> [-inf,inf]</td>
 *      <td> c++ double </td>
 *    </tr>
 *  </table>
 *
 * \section Kinematic Kinematic model of the leg
 *
 * A Kinematic model is also available for the leg, and accessible through
 * amarsi::LegKinematicModel. Both Forward and Inverse model are implemented.
 *
 * Here is a figure depicting the different frame used for this model.
 *
 * \image html KinematicModelReferential.png "Referential definitaion for the provided Kinematic Model"
 *
 */
