/**
 * \defgroup user User Interface
 *
 * The main principle of the user interface is to provide an abstraction layer of
 * the webots API. Indeed the cheetah and its simplified version are complex webots
 * object, that need lot of different webots parts to work together (Supervisor,
 * Controller, physics plug-in). As such an environment is error-prone, it is
 * recommended to use only the libamarsi-quad API, and not the webots one
 * anymore.
 *
 * The interface here should cover all needs for controlling the webots model. Its
 * organized on a small group of classes :
 *  - amarsi::Controller, which functions almost like its webots counter part.
 *    You synchronize the simulation and your controller with the
 *    amarsi::Controller::step() method. and the classes give you access to the
 *    Actuator and \ref sensors.
 *  - amarsi::Supervisor, which give you a control over the simulation :
 *   - "resetting" the simulation, but instead of quitting your controller, it
 *     just performs replace the robot position, and returns its state to normal
 *     one.
 *   - Insure that at each start/reset of the simulation, the physical state of
 *     the robot is correctly initialized.
 *   - Permits to access true simulation variable, like raw Ground Reaction
 *     Force, global position ...
 *   - To restrict the motion of the robot to the sagital plane.
 *   - Performing the benchmark.
 *  - amarsi::Clock that provide information about the simulation time.
 *
 * if you want to now about the model characteristic (Servomotors sensor, ...)
 * please see the \ref model page.
 *
 * \section strucController Structure of a controller.
 *
 * The structure of your controller based on libamarsi-quad is simple. First you
 * need to define an amarsi::Controller object, or a derived one. All the
 * actions that can be performed by the robot are accessible through this class.
 *
 * Then the next step is to initialize the simulation, by calling
 * amarsi::Supervisor::initSimulation(). Indeed, as the robot contains compliant
 * elements, the simulation must be performed for a small amount of time before
 * to start using the robot. This is needed to reach the equilibrium state of
 * these elements.
 * \warning This call must be performed before any call to the Controller
 * interface (but not the constructor which must be called before).
 * If you don't do this, you will either end up with wrong value, or a premature
 * exit of your the main program.
 *
 * \sa \ref BasicControllerExample \ref model
 *
 *
 */
