/*
 * Copyright (c) 2008-2010 Mikaël Mayer
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file         emitter_receiver_robot.hpp
 * \date         15/11/2008
 * \brief         Standard emitter/receiver of a robot, for the Message class.
 * \author       Mikaël Mayer
 */

#ifndef _EMITTER_RECEIVER_ROBOT_HPP_
#define _EMITTER_RECEIVER_ROBOT_HPP_

#include <webots/Robot.hpp>
#include "MessageException.h"
#include "message_private.hpp"

/**
 * \class  EmitterReceiverRobot
 * \author Mikaël Mayer
 * \date   2008/11/15
 * Implements the EmitterReceiver interface
 * with the emitter and receivers of a robot controller.
 *
 * This class can be taken as an example to design another implementation of
 * an EmitterReceiver module.
 *  \ingroup Message
 */
class EmitterReceiverRobot:public EmitterReceiver {
private:
  /// The robot emitter of this module
  webots::Emitter* emitter;

  /// The robot receiver of this module
  webots::Receiver* receiver;

  /// The constructor
  EmitterReceiverRobot(webots::Robot* r) {
    emitter = r->getEmitter("emitter");
    if(!emitter)
      throw MessageException("Inexistant Emitter field named 'emitter'");
    receiver = r->getReceiver("receiver");
    if(!receiver)
      throw MessageException("Inexistant Receiver field named 'receiver'");
    receiver->enable(8);
  }
public:
  /// The default construct procedure
  inline static EmitterReceiver* create(webots::Robot * r) {
    return new EmitterReceiverRobot(r);
  }

  /// Emitter methods
  inline virtual int sendPacket(const void *data, int size) {
    return emitter->send(data, size);
  }

  /// Receiver methods
  inline virtual void enable(unsigned short ms) {
    receiver->enable(ms);
  }
  inline virtual int getQueueLength() {
    return receiver->getQueueLength();
  }
  inline virtual void nextPacket() {
    receiver->nextPacket();
  }
  inline virtual int getDataSize() {
    return receiver->getDataSize();
  }
  inline virtual const void *getData() {
    return receiver->getData();
  }
};

#endif /*_EMITTER_RECEIVER_ROBOT_HPP_*/
