/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Actuator.cpp
 *
 * \date 26 nov. 2009
 * \author Alexandre Tuleu
 */

#include "Actuator.h"
#include <cassert>
#include <iostream>



namespace amarsi {
	Actuator::SetOfActuator Actuator::s_actuators;
	Actuator::Actuator()
	: d_desiredPosition(0.0)
	, d_mode(POSITION_CONTROLLED){
		s_actuators.insert(this);

	}

	Actuator::~Actuator() {
		s_actuators.erase(this);
	}


	void Actuator::updateAllStates(){
		for(SetOfActuator::iterator it = s_actuators.begin();
				it != s_actuators.end();
				++it)
			(*it)->updateState();
	}

	void Actuator::sendAllCommands(){
		for(SetOfActuator::iterator it = s_actuators.begin();
				it != s_actuators.end();
				++it){
			(*it)->sendCommand();
		}
	}

	void Actuator::resetAllActuators(){
		for(SetOfActuator::iterator it = s_actuators.begin();
				it != s_actuators.end();
				++it){
			(*it)->reset();
		}
	}

	Actuator::Mode Actuator::mode() const{
		return d_mode;
	}

	void Actuator::setMode(Actuator::Mode m){
		d_mode = m;
	}
}
