/**
 * \file ServoDriver.cpp
 *
 *  \date Oct 25, 2010
 *  \author tuleu
 */

#include "ServoDriver.h"
#include <libamarsi-quad/controller/Supervisor.h>
namespace amarsi {

	ServoDriver::ServoDriver(webots::Servo* servo)
		: d_servo(servo)
		, d_position(0.0)
		, d_rate(0.0)
		, d_torque(0.0)
		, d_initialized(false)
		, d_timestep(amarsi::Clock::timeStepInS())
		, d_mode(POSITION_DRIVEN){
		if(!d_servo)
			return;
		unsigned int ts(amarsi::Clock::timeStepInMS());
		d_servo->enableMotorForceFeedback(ts);
		d_servo->enablePosition(ts);
	}

	ServoDriver::~ServoDriver(){
	}

	void ServoDriver::update(){
		if(d_initialized){
			d_rate = (d_servo->getPosition() - d_position) / d_timestep;
		}else{
			d_initialized = true;
		}
		d_position = d_servo->getPosition();
		if(d_mode != TORQUE_DRIVEN){
			d_torque = d_servo->getMotorForceFeedback();
		}
	}

	void ServoDriver::sendCommand(double command){
		switch(d_mode){
		case POSITION_DRIVEN :
			d_servo->setPosition(command);
			break;
		case VELOCTY_DRIVEN :
			d_servo->setVelocity(command);
			break;
		case TORQUE_DRIVEN :
			d_servo->setForce(command);
			d_torque = command;
			break;
		default :
			break;
		}
	}
}

