/**
 * \file ServoDriver.h
 *
 *  \date Oct 25, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_QUAD_SERVODRIVER_H_
#define LIBAMARSI_QUAD_SERVODRIVER_H_

#include <webots/Servo.hpp>

/**
 *
 */
namespace amarsi {

	class ServoDriver {
	public:
		enum Mode{
			POSITION_DRIVEN,
			VELOCTY_DRIVEN,
			TORQUE_DRIVEN
		};
		ServoDriver(webots::Servo* servo);
		virtual ~ServoDriver();

		double timeStep() const;

		void setMode(Mode m);

		double position() const;
		double velocity() const;
		double torque() const;
		double power() const;

		void update();
		void reset();
		void sendCommand(double command);

	private :
		webots::Servo* d_servo;
		double d_position;
		double d_rate;
		double d_torque;
		bool d_initialized;
		double d_timestep;
		Mode d_mode;
	};

	inline double ServoDriver::timeStep() const{
		return d_timestep;
	}


	inline double ServoDriver::position() const{
		return d_position;
	}
	inline double ServoDriver::velocity() const{
		return d_rate;
	}
	inline double ServoDriver::torque() const{
		return d_torque;
	}
	inline double ServoDriver::power() const{
		return d_torque * d_rate;
	}
	inline void ServoDriver::setMode(Mode m){
		d_mode = m;
	}
	inline void ServoDriver::reset(){
		d_position = 0.0;
		d_rate     = 0.0;
		d_torque   = 0.0;
		d_initialized = false;
	}
}

#endif // LIBAMARSI_QUAD_SERVODRIVER_H_
