/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file SimpleActuator.h
 *
 * \date 26 nov. 2009
 * \author Alexandre Tuleu
 */

#ifndef LIBAMARSI_QUAD_SIMPLEACTUATOR_H_
#define LIBAMARSI_QUAD_SIMPLEACTUATOR_H_

#include "Actuator.h"
#include "NodeResetter.h"
#include "ServoDriver.h"
namespace amarsi {
	/**
	 * A simple actuator. Work exactly as one from webots, but you get acces
	 * to its consumed power.
	 *
	 * \ingroup internal
	 */
	class SimpleActuator: public Actuator{
	public:
		SimpleActuator(webots::Servo *servo,
		               webots::Node* node);
		virtual ~SimpleActuator();
		virtual void updateState();
		virtual double power() const;
		virtual double torque()const;
		virtual double position() const;
		virtual double velocity() const;
		virtual void sendCommand();
		virtual void reset();
		virtual void setMode(Actuator::Mode m);
	private :
		ServoDriver d_servo;
		NodeResetter d_resetter;

	};
	inline double SimpleActuator::power() const{
		return d_servo.power();
	}
	inline double SimpleActuator::torque()const{
		return d_servo.torque();
	}
	inline double SimpleActuator::position() const{
		return d_servo.position();
	}
	inline double SimpleActuator::velocity() const{
		return d_servo.velocity();
	}
	inline void SimpleActuator::sendCommand(){
		d_servo.sendCommand(order());
	}
	inline void SimpleActuator::reset(){
		d_resetter.reset();
		d_servo.reset();
	}
}

#endif /* SIMPLEACTUATOR_H_ */
