/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Quat.h
 *
 *  \date May 10, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_QUAT_H_
#define LIBAMARSI_QUAT_H_

#include "Vector3.h"

/**
 * Really simple class for Quaternion, to modelize 3D rotations.
 *
 * \ingroup internal
 */

class Quat {
public:

  /// @name Constructors and Destructors
  //@{
  /**
   * Base Constructor.
   * @param setZero if true, will set all coordinate to zero
   */
	Quat(bool setZero=true);
	/**
	 * Create a new Qauternion with given coordiantes.
	 */
	Quat(double w, double x, double y, double z);
	/**
	 * Creates a Quaternion with a zero rotation along the axis defined by the
	 * given Vector.
	 * @param v Vector3 that defines the axis of rotation
	 */
	explicit Quat(const Vector3 &v);
	/**
	 * Creates a Quaternion representing a counter-clockwise rotation defined by
	 * the given axis and angle.
	 * @param v Vector that defines the axis  of the rotation.
	 * @param theta counter-clockwise angle of the rotation
	 */
	Quat(const Vector3 &v, double theta);

	/**
	 * Copy constructor from an Vector<4>.
	 * @param src
	 * @return
	 */
	explicit Quat(const Vector<4> & src);

	virtual ~Quat();
	//@}
	/**
	 * Returns the conjugate of the quaternion.
	 * @return the connjugate.
	 */
	Quat conj() const;
	/**
	 * basic Multuiplication operator.
	 * @param q
	 * @return
	 */
	Quat operator*(const Quat &q) const;
	/**
	 * Rotate a Vector3 with the rotation deifned by the Quaternion.
	 * @param v
	 * @return
	 */
	Vector3 rotate(const Vector3 & v) const;
private :

	double w_,x_,y_,z_;

};

#endif // libamarsi_QUAT_H_
