/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Vector3.h
 *
 *  \date May 10, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_VECTOR3_H_
#define LIBAMARSI_VECTOR3_H_

#include "Vector.h"

/**
 * A 3D Vectors with its extended operation (against standard N-dimensional
 * Vector)
 *
 * \ingroup internal
 *
 */
class Vector3 : public Vector<3>{
public:
  /// @name standard constructor.
  //@{
	Vector3(bool zero=true);
	explicit Vector3(const double *a);
	template <uint NO> explicit Vector3(const Vector<NO> & src);
	Vector3(const Vector3 & src);

	const Vector3 & operator=(const Vector<3> & src);

	Vector3(double dx, double dy, double dz){
		_[0]=dx;_[1]=dy;_[2]=dz;
	}
	virtual ~Vector3();
	//@}

	/**
	 * Computes the vectorial product of two vector3.
	 */
	void prodVect(const Vector3 & other, Vector3 & result) const;
	/**
	 * Computes the vectorial product of two Vector3.
	 */
	friend Vector3 operator^(const Vector3 &a, const Vector3 &b);

	///@name Read and read/Write accessors.
	//@{
	double cx() const{
		return _[0];
	}

	double cy() const{
		return _[1];
	}

	double cz() const{
		return _[2];
	}
	double& x(){
		return _[0];
	}

	double& y(){
		return _[1];
	}

	double& z(){
		return _[2];
	}
	//@}

	///@name Predefined Vector3
  //@{
	static const Vector3 ZERO;
	static const Vector3 EX;
	static const Vector3 EY;
	static const Vector3 EZ;
	//@}
};
template <uint NO> Vector3::Vector3(const Vector<NO> & src)
 : Vector<3>(src){
}
#endif // libamarsi_VECTOR3_H_
