/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file VectorTest.cpp
 *
 *  \date May 11, 2010
 *  \author tuleu
 */

#include "VectorTest.h"

/**
 * \test EqualityBetweenVectors
 * tests the equality between two vector that are the same.
 */
TEST_F(VectorTest,EqualityBetweenVectors){
	EXPECT_EQ(a,b);
}
/**
 * \test NewVectorAreZeroByDefault
 *
 * verifies that a new unitialized Vector<N> is equal to Vector<N>::Zero
 *
 */
TEST_F(VectorTest,NewVectorAreZeroByDefault){
	EXPECT_EQ(Vector<4>(),Vector<4>::ZERO);
}

TEST_F(VectorTest,AdditionOfVectors){
	Vector<4> e;
	a.add(b,e);
	EXPECT_FLOAT_EQ(2.0,e[0]);
	EXPECT_FLOAT_EQ(4.0,e[1]);
	EXPECT_FLOAT_EQ(6.0,e[2]);
	EXPECT_FLOAT_EQ(8.0,e[3]);

	e=b;
	e+=2.5;

	EXPECT_FLOAT_EQ(3.5,e[0]);
	EXPECT_FLOAT_EQ(4.5,e[1]);
	EXPECT_FLOAT_EQ(5.5,e[2]);
	EXPECT_FLOAT_EQ(6.5,e[3]);
}


TEST_F(VectorTest,MultiplicationOfVector){
	Vector<4> e;
	a.mult(2.0,e);
	EXPECT_EQ(e,a+b);
}


TEST_F(VectorTest,ScalarProductIdentities){
	EXPECT_FLOAT_EQ(a*a,a.norm2());
	EXPECT_FLOAT_EQ(0.0,b*c);
	EXPECT_FLOAT_EQ(std::sqrt(2),d.norm());
	EXPECT_FLOAT_EQ(0.0,d*Vector<4>::E(0));
}

TEST_F(VectorTest, CopyOfVectorFromDifferentDimension){
	Vector<2> a2res;
	a2res[0]=1;
	a2res[1]=2;
	Vector<6> a6res;
	a6res[0]=1;
	a6res[1]=2;
	a6res[2]=3;
	a6res[3]=4;
	a6res[4]=0;
	a6res[5]=0;

	Vector<2> test2(a);
	Vector<6> test6(a);
	EXPECT_EQ(a2res,test2);
	EXPECT_EQ(a6res,test6);
}
