/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file AtomicTestQuad.cpp
 *
 *  \date Aug 24, 2010
 *  \author tuleu
 */
#include <libamarsi-quad/controller/Supervisor.h>

#include "AtomicTestQuad.h"

namespace benchmark {

  AtomicTestQuad::AtomicTestQuad()
  : d_duration(0.0){

  }

  AtomicTestQuad::~AtomicTestQuad(){
  }

  void AtomicTestQuad::setUpPimp(){
    internalInputs("measurement-time",d_duration);
    s_supervisorLockKey.unlock();
    amarsi::Supervisor::instance().configureMotionRestriction("0.0:FREE");
    setUpPimpQuad();
    s_supervisorLockKey.lock();
  }

  void AtomicTestQuad::tearDownPimp(){
    s_supervisorLockKey.unlock();
    tearDownPimpQuad();
    s_supervisorLockKey.lock();
  }


  void AtomicTestQuad::setSupervisorLock(const amarsi::Lock::Key & supervisorLock){
    s_supervisorLockKey = supervisorLock;

    s_supervisorLockKey.lock();
  }

  amarsi::Lock::Key AtomicTestQuad::s_supervisorLockKey(amarsi::Lock::Key::getAnInvalidKey());

  bool AtomicTestQuad::isStillValid(){
    s_supervisorLockKey.unlock();
    bool ret = isStillValidQuad();
    if(ret){
      s_supervisorLockKey.lock();
    }
    //if we will stop, then we authorize modification. Then if it is a repeated,
    // tearDown will one called and relock everything
    //if its not, then simulationQuit will be called then we are safe, the user
    //will not be able to perform modification

    return ret;
  }

  bool AtomicTestQuad::isStandingUp(){
    const amarsi::State & state(amarsi::Supervisor::instance().state());

    if(state.hasABadContact)
      return false;
    double prodScal = state.startingUpDirection * state.upDirection;
    if(prodScal < 0.0 )//cos(80°)
      return false;
    return true;
  }

  double AtomicTestQuad::distance() const{
    const amarsi::State & state(amarsi::Supervisor::instance().state());

    return state.position[0]-state.startingPosition[0];

  }

}
