/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file TestCheetahPushes.cpp
 *
 *  \date Sep 1, 2010
 *  \author tuleu
 */

#include <libamarsi-quad/controller/Supervisor.h>

#include "TestCheetahPushes.h"

namespace benchmark {

  TestCheetahPushes::TestCheetahPushes()
  : d_perturbated(false)
  , d_startTime(0.0)
  , d_pertDuration(0.0)
  , d_amplitude(0.0){

  }

  TestCheetahPushes::~TestCheetahPushes(){
  }


  void TestCheetahPushes::computeOutputsPimp(){
  	const amarsi::State & state= amarsi::Supervisor::instance().state();

  	Vector3 fDirection(state.frontDirection);
  	fDirection.y()=0.0;
  	fDirection.normalize();

  	outputs()["average-speed"] = ((state.position
                                   - state.startingPosition))*fDirection/duration();
  	outputs()["is-standing"] = isStandingUp() ? 1.0 : 0.0;

   }

   void TestCheetahPushes::setUpPimpQuad(){
     internalInputs("perturbation-start-time",d_startTime);
     internalInputs("perturbation-duration",d_pertDuration);
     internalInputs("perturbation-amplitude",d_amplitude);
   }
   void TestCheetahPushes::tearDownPimpQuad(){

   }
   bool TestCheetahPushes::isStillValidQuad(){
     if(amarsi::Clock::getTime()>=duration()+d_startTime || !isStandingUp()){
       return false;
     }
     if(!d_perturbated && amarsi::Clock::getTime() >= d_startTime){
       d_perturbated = true;
       applyPerturbation();
     }

     return true;
   }

}
