/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file WebotsSupervisorQuad.cpp
 *
 *  \date Aug 3, 2010
 *  \author tuleu
 */
#include "benchmarks.h"
#include "WebotsSupervisorQuad.h"
#include <libamarsi-quad/controller/Supervisor.h>
#include <libamarsi-quad/utils/Debug.h>

#define REGISTER_ATOMIC_TEST(TestName) TestName tmp ## TestName;\
  Factory<benchmark::AtomicTest>::registerObject(tmp ## TestName.name(),tmp ## TestName);

namespace benchmark {

  WebotsSupervisorQuad::WebotsSupervisorQuad(){
  }

  WebotsSupervisorQuad::~WebotsSupervisorQuad(){

  }

  void WebotsSupervisorQuad::initialize(){
    if(WebotsSupervisorQuad::s_initialized)
      return;
    //never go back here
    WebotsSupervisorQuad::s_initialized = true;


    //Register all the atomic tests
    REGISTER_ATOMIC_TEST(TestTrivialLocomotion)
    REGISTER_ATOMIC_TEST(TestCheetahPushesSagital)
    REGISTER_ATOMIC_TEST(TestCheetahPushes3D)
    REGISTER_ATOMIC_TEST(TestFrontSlope)
    REGISTER_ATOMIC_TEST(TestSideSlope)
    REGISTER_ATOMIC_TEST(TestFrontStep)


    //initialize the instance
    std::auto_ptr<WebotsSupervisor> instance(new WebotsSupervisorQuad());
    if(instance->isExternallyDriven()){
      AtomicTestQuad::setSupervisorLock(amarsi::Supervisor::instance().lock());
      debug_supervisor("The simulator is externally driven by GUI.");
    }else{
      debug_supervisor("The simulator is not driven by GUI.");
    }

    WebotsSupervisor::initialize(instance);

  }

  bool WebotsSupervisorQuad::s_initialized(false);

  void WebotsSupervisorQuad::quitCommand(){
    amarsi::Supervisor::instance().simulationQuit(0);
  }

}
