/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file WebotsExplorer.cpp
 *
 * \date 16 oct. 2009
 * \author Alexandre Tuleu
 */

#include "libamarsi-quad/common/WebotsExplorer.h"
#include <assert.h>
#include <iostream>

using namespace std;
using namespace webots;
namespace WebotsExplorer{
  bool getField(const webots::Node* node,
                const std::string &name,
                webots::Field ** retField){
    if(!node)
      return false;
    webots::Field* test=node->getField(name);
    if(test==NULL){
      cerr<<"Error : field "<<name<<" doesn't exist"<<endl;
      return false;
    }
    *retField = test;
    //cout<<"Got a new field. Its type is "<<(*retField)->getType()<<endl;
    return true;
  }

  bool getChildren(const webots::Node* node,
                   std::vector<webots::Node*> &retChilds){
    return getChildren(node,0,retChilds);
  }
  bool getChildren(const webots::Node* node, unsigned int minChild,
                   std::vector<webots::Node*> &retChilds){
      Field* childs;
      if(!getField(node,"children",&childs))
        return false;

      retChilds.clear();
      retChilds.assign(childs->getCount(),static_cast<webots::Node*>(NULL));

      for(unsigned int i =0;i<retChilds.size();++i){
        retChilds[i]=childs->getMFNode(i);
      }
      return retChilds.size()>=minChild;
  }

  bool getBoxSizeFromTransform(const webots::Node* node,webots::Node*& retNode){
    Field* childs;

    if(!getField(node,"children",&childs))
      return false;

    if(childs->getCount()!=1)
      return false;

    Node* shape(childs->getMFNode(0));
    Field* geometry;
    if(!getField(shape,"geometry",&geometry))
      return false;
    retNode=geometry->getSFNode();
    return true;
  }
}
