/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file WebotsExplorer.h
 *
 * \date 16 oct. 2009
 * \author Alexandre Tuleu
 */
#ifndef _WEBOTS_EXPORER_H
#define _WEBOTS_EXPORER_H

#include <webots/Node.hpp>
#include <vector>

/**
 * Encapsulates global function that explore a Webots VRML tree.
 *
 * \ingroup internal
 *
 */

namespace WebotsExplorer {
  /**
   * Gets a webots::Field given a Node and a name.
   * @param node the webots::Node to explore
   * @param name the name of the wanted webots::Field
   * @param retField the returned field.
   * @return \c true if successfully found the requested webots::Field.
   */
  bool getField( const webots::Node* node,
                const std::string &name,
                webots::Field** retField);
  /**
   * Gets the childrens of a given Node.
   * @param node the webots::Node to explore
   * @param minChild the expected number of child to found.
   * @param retChilds the list of found webots::Node, will be cleared before the
   * parse, so will contains only the found childrens.
   * @return \c true if found at least minChild childrens.
   */
  bool getChildren(const webots::Node* node, unsigned int minChild,
                   std::vector<webots::Node*> &retChilds);

  /**
   * Gets the childrens of a given Node.
   * @param node the webots::Node to explore
   * @param retChilds the list of found webots::Node, will be cleared before the
   * parse, so will contains only the found childrens.
   * @return always \c true;
   */
  bool getChildren(const webots::Node* node,
                   std::vector<webots::Node*> &retChilds);

  /**
   * Gets the size of a Box field, given the ox node.
   * @param node the Box node
   * @param retNode the size node.
   * @return \c true iff successfuly parsed.
   */
  bool getBoxSizeFromTransform(const webots::Node* node,
                               webots::Node** retNode);
}

#endif//_WEBOTS_EXPORER_H
