/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file common/global.h
 *
 * \date 16 oct. 2009
 * \author Alexandre Tuleu
 */
#ifndef _CHEETAH_GLOBAL_H
#define _CHEETAH_GLOBAL_H

#include <sstream>
//#include <vector>
//#include <string>

/*!
 * Base namespace of the libamarsi-quad, for avoiding name conflict. Please notice
 * that "utilitary" classes such as Debug and PreferenceReader, and filetring
 * tools are not in this namespace.
 */
namespace amarsi {

  /**
   * The differnet Limbs of the robot.
   * \ingroup user
   */
  enum Limbs {
    NONE=-1,
    LEFT_FORE = 0, //!< LEFT_FORE
    RIGHT_FORE = 1,//!< RIGHT_FORE
    LEFT_HIND = 2, //!< LEFT_HIND
    RIGHT_HIND = 3, //!< RIGHT_HIND
    NUMBER_OF_LIMBS = 4
  };


    /**
     * The different ActuatorID accessible for the model.
     * User should use the amarsi::Controller::actuator() method top access
     * Actuator.
     *
     * \sa amarsi::Actuator amarsi::Controller::actuator()
     *
     * \ingroup user
     */
    enum ActuatorID {
      LEFT_FORE_SCAPULA = 0, //!< LEFT_FORE_SCAPULA disabled, here for eventual improvement
      RIGHT_FORE_SCAPULA,    //!< RIGHT_FORE_SCAPULA disabled, here for eventual improvement
      LEFT_HIND_SCAPULA,     //!< LEFT_HIND_SCAPULA disabled, here for eventual improvement
      RIGHT_HIND_SCAPULA,    //!< RIGHT_HIND_SCAPULA disabled, here for eventual improvement
      LEFT_FORE_HIP,         //!< LEFT_FORE_HIP Actuator that drive the left fore hip joint.
      RIGHT_FORE_HIP,        //!< RIGHT_FORE_HIP Actuator that drive the right fore hip joint.
      LEFT_HIND_HIP,         //!< LEFT_HIND_HIP Actuator that drive the left hind hip joint.
      RIGHT_HIND_HIP,        //!< RIGHT_HIND_HIP Actuator that drive the right hind hip joint.
      LEFT_FORE_KNEE,        //!< LEFT_FORE_KNEE Actuator that drive the left fore knee joint.
      RIGHT_FORE_KNEE,       //!< RIGHT_FORE_KNEE Actuator that drive the right fore knee joint.
      LEFT_HIND_KNEE,        //!< LEFT_HIND_KNEE Actuator that drive the left hind knee joint.
      RIGHT_HIND_KNEE,       //!< RIGHT_HIND_KNEE Actuator that drive the right hind knee joint.
      LEFT_FORE_TOE,         //!< LEFT_FORE_TOE disabled, here for eventual improvement
      RIGHT_FORE_TOE,        //!< RIGHT_FORE_TOE disabled, here for eventual improvement
      LEFT_HIND_TOE,         //!< LEFT_HIND_TOE disabled, here for eventual improvement
      RIGHT_HIND_TOE,        //!< RIGHT_HIND_TOE disabled, here for eventual improvement
      SPINAL_COORD_SAGITAL,  //!< SPINAL_COORD_SAGITAL disabled, here for eventual improvement
      SPINAL_COORD_TRANSVERSE,//!< SPINAL_COORD_TRANSVERSE disabled, here for eventual improvement
      NUMBER_OF_ACTUATORS
    };

    inline static Limbs getLimbs(ActuatorID id);

    /**
     * DisplayMode flags for the rendering of the simulation
     */
    enum DisplayMode{
          NO_DISPLAY=0x00,                 //!< NO_DISPLAY don't display visual hint
          SHOW_GROUND_REACTION_FORCES=0x01,//!< SHOW_GROUND_REACTION_FORCES show the Ground Reactions forces
          USE_PRETTY_ARROW = 0x02 ,        //!< USE_PRETTY_ARROW uses a better drawed arrow (not completely functionnal yet)
          SHOW_PERTURBATION_FORCE = 0x04   //!< SHOW_PERTURBATION_FORCE shows the perturbation force applied to the robot.
    };


    /*,
    LEFT_FORE_KNEE = 4,
    RIGHT_FORE_KNEE = 5,
    LEFT_HIND_KNEE=6,
    RIGHT_HIND_KNEE = 7,
    LEFT_FORE_TOE= 8,
    RIGHT_FORE_TOE = 9,
    LEFT_HIND_TOE=10,
    RIGHT_HIND_TOE = 11,
    NONE=12
    };*/
//std::vector<std::string> LimbsName;

/*  class LegDescriptor{
    double length_, kFemur_, kTibia_, kFoot_, deltaTibia_,x_,y_,z_;
    bool left_;
  public :
    LegDescriptor(double length, double kFemur, double kTibia, double kFoot, double deltaTibia_,
                  double x=0.0, double y=0.0, double z=0.0, bool left=true);
    virtual ~LegDescriptor();

    double getX()const{
      return x_;
    }
    double getY()const{
      return y_;
    }
    double getZ()const{
      return z_;
    }

    bool isLeft()const{
      return left_;
    }

    double getLength() const{
      return length_;
    }

    double getFemurLength() const{
      return kFemur_*length_+deltaTibia_/2;
    }

    double getFemurCoefficient() const{
      return kFemur_;
    }

    double getTibiaLength() const{
      return kTibia_*length_;
    }

    double getTibiaCoefficient() const{
      return kTibia_;
    }

    double getFootLength() const{
      return kFoot_*length_+deltaTibia_/2;
    }

    double getFootCoefficient() const{
      return kFoot_;
    }

    double getDeltaTibia() const{
      return deltaTibia_;
    }
  };*/
    inline Limbs getLimbs(ActuatorID id){
    if(id>RIGHT_HIND_TOE)
      return NONE;
    return static_cast<Limbs>(id%4);
  }
}


#endif //_CHEETAH_GLOBAL_H
