/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Clock.h
 *
 * \date 18 déc. 2009
 * \author Alexandre Tuleu
 */

#ifndef CLOCK_H_
#define CLOCK_H_

namespace amarsi {
	/**
	 * The Clock of our simulation. Its differet of the webots::Clock, as we allow
	 * to perform several time the same simulation during one global simulation.
	 * So if you design a controller that depends on time, please use this class
	 * to access time value.
	 * This clock could be reseted by derived classes through resetClock. So
	 * don't use it for computation, but for triggering events.
	 *
	 * \ingroup user
	 *
	 * \todo this design is a flaw since you can make infinite loop : if you
	 * call a resetClock during a loop whose condition is time. So it may be good
	 * to introduce a lock() mechanism_ to avoid that.
	 *
	 */
	class Clock {
	private:
		static double s_timeEllapsed;
		static double s_timestepInS;
		static unsigned int s_timestepInMS;
	public:
		Clock();
		virtual ~Clock();
		/**
		 * Returns the actual time.
		 * \attention This time is not the webots time, because a Clock can ever
		 * reseted to another time.
		 * the time to 0.
		 * @return the actual time.
		 */
		static double getTime() {
			return s_timeEllapsed;
		}
		/**
		 * Returns the basic time step of the simulation in seconds.
		 * * @return the time step in seconds.
		 */
		static double timeStepInS(){
			return s_timestepInS;
		}
		/**
		 * Returns the basic time step of the simulation in milliseconds.
		 * * @return the time step in milliseconds.
		 */
		static unsigned int timeStepInMS(){
			return s_timestepInMS;
		}
	protected :
		/**
		 * Time modificator for derivated classes.
		 * @return
		 */
		double& clockTime() {
			return s_timeEllapsed;

		};
		void setTimeStep(unsigned int ts){
			s_timestepInMS = ts;
			s_timestepInS  = ((double) ts) / (1000.0);
		}
	};
}
#endif /* CLOCK_H_ */
