/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Controller.cpp
 *
 *  \date May 11, 2010
 *  \author tuleu
 */

#include <stdlib.h>

#include <libamarsi-quad/common/argstream.h>
#include <libamarsi-quad/utils/PreferenceReader.h>
#include <libamarsi-quad/utils/Debug.h>
#include <libamarsi-quad/actuators/Actuator.h>
#include <libamarsi-quad/sensors/Sensors.h>
#include <libamarsi-quad/controller/internals/ControllerPrimple.h>
#include <libamarsi-quad/controller/Robot.h>


#include "EventManager.h"
#include "Supervisor.h"

#include "Controller.h"
#include "Robot.h"

namespace amarsi {

  ControllerPrimple * Controller::pimpController_(NULL);

  Controller::Controller(int& argc, char** argv){
    initController(argc,argv);
  }

  Controller::~Controller() {
  }
  void Controller::actionsEverySecond(){
    this->userActionEverySecond();
  }


  void Controller::userInitSimulation(){

  }
  void Controller::userInitEnvironment(){

  }

  void Controller::userActionEverySecond(){

  }


  void Controller::initController(int & argc, char ** argv){
    //create a Supervisor if it was not created
    Supervisor::instance();

    argstream as(argc,argv);

    std::string prefFile;

    as>>parameter('p',"pref",prefFile,"The preference file to read",false)
      >>help();
    as.defaultErrorHandling();

    if(!prefFile.empty())
      PreferenceReader::instance().read(prefFile);


    //will create our pimpcontroller if not already created
    Supervisor::instance().setSimulationController(this);
  }


  Actuator& Controller::actuator(ActuatorID id) const{
     return *(Controller::pimpController_->actuators()[id]);
  }

  void Controller::step(uint multiple){
    Supervisor::checkIfPhysicsIsInitialized();
    Controller::pimpController_->step((multiple==0?1:multiple)*Clock::timeStepInMS());
  }
  const PositionEncoder& Controller::kneeEncoder(Limbs l) const {
    if(l == NONE)
      l=LEFT_FORE;/// \todo should throw exception
    return *(Controller::pimpController_->kneeEncoders()[l]);
  }
  const Touch3DSensor& Controller::touchSensor(Limbs l) const {
    if(l == NONE)
      l=LEFT_FORE;/// \todo should throw exception
    return *(Controller::pimpController_->touchSensors()[l]);
  }
  const Gyro & Controller::gyro() const {
    return *(Controller::pimpController_->gyro());
  }
  const Accelerometer & Controller::accelerometer() const {
    return *(Controller::pimpController_->accelerometer());
  }


  amarsi::Robot * Controller::robot(){
    return pimpController_->robot();
  }
}



