/**
 * \file Robot.h
 *
 *  \date Sep 22, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_QUAD_ROBOT_H_
#define LIBAMARSI_QUAD_ROBOT_H_

#include <webots/Robot.hpp>
#include <set>

#include <libamarsi-quad/utils/Lock.h>
/**
 *
 */
namespace amarsi {

  class Robot {

  public:
    Robot(webots::Robot *robot);
    virtual ~Robot();

    webots::Accelerometer *getAccelerometer(const std::string &name);
    webots::Camera *getCamera(const std::string &name);
    webots::Compass *getCompass(const std::string &name);
    webots::Connector *getConnector(const std::string &name);
    webots::Display *getDisplay(const std::string &name);
    webots::DistanceSensor *getDistanceSensor(const std::string &name);
    webots::Emitter *getEmitter(const std::string &name);
    webots::GPS *getGPS(const std::string &name);
    webots::Gyro *getGyro(const std::string &name);
    webots::LED *getLED(const std::string &name);
    webots::LightSensor *getLightSensor(const std::string &name);
    webots::Pen *getPen(const std::string &name);
    webots::Receiver *getReceiver(const std::string &name);
    webots::Servo *getServo(const std::string &name);
    webots::TouchSensor *getTouchSensor(const std::string &name);
    int getMode() const;
    const std::string getName() const;
    const std::string getProjectPath() const;
    bool getSynchronization() const;
    double getTime() const;


    void registerForbiddenName(const std::string & name);

  private :
    typedef std::set<std::string> SetOfName;

    webots::Robot *d_robot;

    SetOfName d_forbiddenName;


  };

}

#endif // LIBAMARSI_QUAD_ROBOT_H_
