/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file State.cpp
 *
 * \date 18 dec. 2009
 * \author Alexandre Tuleu
 */

#include "State.h"
#include "Supervisor.h"

namespace amarsi{
  State::State()
    : instantaneousSpeed(0.0)
    , hasABadContact(false)
    , consumedEnergy(0.0)
    , upDirectionOwn_(Vector3::EY)
    , frontDirectionOwn_(Vector3::EX)
    , sideDirectionOwn_(Vector3::EZ){
    GRFs.assign(4,Vector3::ZERO);
  }


  void State::reset(){
    GRFs.assign(GRFs.size(),Vector3::ZERO);
    startingPosition=position;
    previousPosition=position;
    startingOrientation=orientation;
    startingUpDirection=upDirection;
    startingFrontDirection=frontDirection;
    startingSideDirection=sideDirection;
    hasABadContact=false;
    consumedEnergy=0.0;
  }

  void State::updateFields(const Vector3 & newPosition,
                           const Quat& newOrientation){

  	orientation=newOrientation;
    frontDirection=orientation.rotate(frontDirectionOwn_);
    upDirection=orientation.rotate(upDirectionOwn_);
    sideDirection=orientation.rotate(sideDirectionOwn_);

    if(previousPosition.norm2()>0){
      instantaneousSpeed=(newPosition-position).norm()/Clock::timeStepInS();
    }else
      instantaneousSpeed=0.0;

    previousPosition=position;
    position = newPosition;

  }

  const LegKinematicModel & State::legModel(amarsi::Limbs l) const {
    if(l == NONE || l == NUMBER_OF_LIMBS)
      return *(kinematicModels_[0]);/// \todo should throw exception
    return *(kinematicModels_[l]);
  }
}


