/*
 * Copyright (c) 2010 Ecole Polytechnique Fédérale de Lausanne
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file PimpController.h
 *
 *  \date May 11, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_PIMPCONTROLLER_H_
#define LIBAMARSI_PIMPCONTROLLER_H_
#include <ctime>

//#include <webots/Supervisor.hpp>
#include <libamarsi-quad/controller/State.h>
#include <libamarsi-quad/controller/Clock.h>

//A forward declaration of amarsi::Sensor causes a strange
//compilation time error, only mean to overcome this
//#include <libamarsi-quad/sensors/Sensors.h>

#include <libamarsi-quad/basic-math/Vector3.h>

class Message;

namespace webots{
  class Supervisor;
  class Node;
  class Field;
}

namespace amarsi {
  class Controller;
  class Actuator;
  class MotionRestrictorManager;
  class Supervisor;
  class PositionEncoder;
  class Touch3DSensor;
  class Gyro;
  class Accelerometer;
  class Robot;
  /**
   * \defgroup internal Internal Module
   *
   * All the classes listed here are internal of libamarsi-quad. End user may not
   * be interested-in, but they may be useful to hack or to build new controllers.
   */
  //@{
  /**
   * Pimped classes for controller
   *
   * \todo : initialization processus should be in OOP !!!!
   */
  class ControllerPrimple : public Clock {
    friend class amarsi::Supervisor;
  public:
    typedef std::vector<amarsi::Actuator*> SetOfActuator;


    ControllerPrimple(webots::Supervisor *supervisor);
    virtual ~ControllerPrimple();

    virtual int step(int millisec);
    const SetOfActuator& actuators() const;
    const std::vector<PositionEncoder*> & kneeEncoders() const;
    const std::vector<Touch3DSensor*> & touchSensors() const;
    const Gyro * gyro() const;
    const Accelerometer * accelerometer() const;

    void resetAllOrders() const;
    void preRunPhysics(double seconde);
    void resetRobotPosition() const;
    void simulationQuit();
    void simulationQuit(unsigned int returnCode);

    void setTrackingOn(bool on=true);


    amarsi::Robot * robot();

  protected :
    void checkForNewMessages();

    virtual void handleNewMessage(Message & m);

    virtual void updateGlobals();
  protected :
    virtual void initActuators()=0;
    virtual void initSensors()=0;
    virtual void initModels()=0;
    /**
     * Tells physics plug-in to desactivate the MotionRestrictor (FREE state).
     * This method will call a physics step needed to transmit the message to,
     * the physics plug-in, and then remove the constraint, but
     * simulation time provided by Clock::getTime() will stay the same.
     */
    void disableMotionRestrictor();

    void init(webots::Node* robotNode);

    void initMessages();

    void getCameraField();

    static bool messagesEnabled_;

    Controller * userController_;
    webots::Supervisor *supervisor_;
    amarsi::Robot *robot_;
    SetOfActuator actuators_;
    std::vector<amarsi::PositionEncoder*> kneeEncoders_;
    std::vector<amarsi::Touch3DSensor*> touchSensors_;
    Gyro* gyro_;
    Accelerometer* accelerometer_;


    unsigned int seconds_;

    State state_;

    MotionRestrictorManager* mrManager_;


    webots::Field* position_;
    webots::Field* orientation_;

    Vector3 initialPosition_;
    Vector<4> initialOrientation_;

    clock_t start_,end_;
    bool enableProfiling_;

    webots::Field* d_cameraView;
    Vector3 d_diffView;
    Vector3 d_cameraPosition;
    double d_heightToConserve;
    bool d_trackingEnabled;


  };
  //@}


//==============================================================================
// INLINE IMPLEMENTATION
//==============================================================================

  inline
  const std::vector<PositionEncoder*> & ControllerPrimple::kneeEncoders() const{
    return kneeEncoders_;
  }

  inline
  const std::vector<Touch3DSensor*> & ControllerPrimple::touchSensors() const{
    return touchSensors_;
  }

  inline
  const Gyro * ControllerPrimple::gyro() const{
    return gyro_;
  }

  inline
  const Accelerometer * ControllerPrimple::accelerometer() const{
    return accelerometer_;
  }

}
#endif // LIBAMARSI_PIMPCONTROLLER_H_
//include inline complex inline function
#include "ControllerPrimple-inl.h"
