/*
 * Copyright (c) 2010 Ecole Polytechnique Fédérale de Lausanne
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file PimpControllerAmarsi.cpp
 *
 *  \date May 21, 2010
 *  \author tuleu
 */

#include <webots/Supervisor.hpp>

#include <libamarsi-quad/common/global.h>
#include <libamarsi-quad/common/WebotsExplorer.h>
#include <libamarsi-quad/actuators/Actuators.ih>
#include <libamarsi-quad/sensors/Sensors.h>
#include <libamarsi-quad/model/model-private.h>
#include <libamarsi-quad/controller/Robot.h>

#include "ControllerPrimpleAmarsi.h"

namespace amarsi {

  ControllerPrimpleAmarsi::ControllerPrimpleAmarsi(webots::Supervisor *supervisor,
                                             webots::Node * robotNode)
    : ControllerPrimple(supervisor){
    init(robotNode);
  }

  ControllerPrimpleAmarsi::~ControllerPrimpleAmarsi() {

  }

  void ControllerPrimpleAmarsi::initActuators(){
    actuators_.assign(NUMBER_OF_ACTUATORS,NULL);

    actuators_[LEFT_FORE_HIP] =
        new SimpleActuator(supervisor_->getServo("LEFT_FORE_hip_servo"),
                           supervisor_->getFromDef("LEFT_FORE_HIP_SERVO"));

    actuators_[RIGHT_FORE_HIP] =
        new SimpleActuator(supervisor_->getServo("RIGHT_FORE_hip_servo"),
                           supervisor_->getFromDef("RIGHT_FORE_HIP_SERVO"));

    actuators_[LEFT_HIND_HIP] =
        new SimpleActuator(supervisor_->getServo("LEFT_HIND_hip_servo"),
                           supervisor_->getFromDef("LEFT_HIND_HIP_SERVO"));
    actuators_[RIGHT_HIND_HIP] =
        new SimpleActuator(supervisor_->getServo("RIGHT_HIND_hip_servo"),
                           supervisor_->getFromDef("RIGHT_HIND_HIP_SERVO"));


    actuators_[LEFT_FORE_KNEE] =
        new KneeActuatorAmarsi(supervisor_->getServo("LEFT_FORE_ANKLE_SERVO"),
                               supervisor_->getServo("LEFT_FORE_FRONT_KNEE_SERVO"),
                               supervisor_->getFromDef("LEFT_FORE_FRONT_KNEE"),
                               supervisor_->getFromDef("LEFT_FORE_ANKLE"),
                               LEFT_FORE);
    actuators_[RIGHT_FORE_KNEE] =
        new KneeActuatorAmarsi(supervisor_->getServo("RIGHT_FORE_ANKLE_SERVO"),
                               supervisor_->getServo("RIGHT_FORE_FRONT_KNEE_SERVO"),
                               supervisor_->getFromDef("RIGHT_FORE_FRONT_KNEE"),
                               supervisor_->getFromDef("RIGHT_FORE_ANKLE"),
                               RIGHT_FORE);
    actuators_[LEFT_HIND_KNEE] =
        new KneeActuatorAmarsi(supervisor_->getServo("LEFT_HIND_ANKLE_SERVO"),
                               supervisor_->getServo("LEFT_HIND_FRONT_KNEE_SERVO"),
                               supervisor_->getFromDef("LEFT_HIND_FRONT_KNEE"),
                               supervisor_->getFromDef("LEFT_HIND_ANKLE"),
                               LEFT_HIND);
    actuators_[RIGHT_HIND_KNEE] =
        new KneeActuatorAmarsi(supervisor_->getServo("RIGHT_HIND_ANKLE_SERVO"),
                               supervisor_->getServo("RIGHT_HIND_FRONT_KNEE_SERVO"),
                               supervisor_->getFromDef("RIGHT_HIND_FRONT_KNEE"),
                               supervisor_->getFromDef("RIGHT_HIND_ANKLE"),
                               RIGHT_HIND);

    //now lot of void actuators

    actuators_[LEFT_FORE_SCAPULA]=new VoidActuator();
    actuators_[RIGHT_FORE_SCAPULA]=new VoidActuator();
    actuators_[LEFT_HIND_SCAPULA]=new VoidActuator();
    actuators_[RIGHT_HIND_SCAPULA]=new VoidActuator();

    actuators_[LEFT_FORE_TOE]=new VoidActuator();
    actuators_[RIGHT_FORE_TOE]=new VoidActuator();
    actuators_[LEFT_HIND_TOE]=new VoidActuator();
    actuators_[RIGHT_HIND_TOE]=new VoidActuator();

    actuators_[SPINAL_COORD_SAGITAL]=new VoidActuator();
    actuators_[SPINAL_COORD_TRANSVERSE]=new VoidActuator();

  }

  void ControllerPrimpleAmarsi::initSensors(){
      kneeEncoders_.assign(4,(amarsi::PositionEncoder*)NULL);
      touchSensors_.assign(4,(amarsi::Touch3DSensor*)NULL);


      kneeEncoders_[LEFT_FORE ] =
          new PositionEncoder(supervisor_->getServo("LEFT_FORE_FRONT_KNEE_SERVO"),
                              Clock::timeStepInMS(),0,3.14159265,12,true);
      kneeEncoders_[RIGHT_FORE] =
          new PositionEncoder(supervisor_->getServo("RIGHT_FORE_FRONT_KNEE_SERVO"),
                              Clock::timeStepInMS(),0,3.14159265,12,true);
      kneeEncoders_[LEFT_HIND]  =
          new PositionEncoder(supervisor_->getServo("LEFT_HIND_FRONT_KNEE_SERVO"),
                              Clock::timeStepInMS(),0,3.14159265,12,true);
      kneeEncoders_[RIGHT_HIND] =
          new PositionEncoder(supervisor_->getServo("RIGHT_HIND_FRONT_KNEE_SERVO"),
                              Clock::timeStepInMS(),0,3.14159265,12,true);

      touchSensors_[LEFT_FORE] =
          new Touch3DSensor(state_.GRFs[LEFT_FORE],0.5,0.0,0.0,100.0,12);
      touchSensors_[RIGHT_FORE] =
          new Touch3DSensor(state_.GRFs[RIGHT_FORE],0.5,0.0,0.0,100.0,12);
      touchSensors_[LEFT_HIND] =
          new Touch3DSensor(state_.GRFs[LEFT_HIND],0.5,0.0,0.0,100.0,12);
      touchSensors_[RIGHT_HIND] =
          new Touch3DSensor(state_.GRFs[RIGHT_HIND],0.5,0.0,0.0,100.0,12);

      gyro_=new Gyro(supervisor_->getGyro("gyro"),Clock::timeStepInS());

      robot_->registerForbiddenName("gyro");

      webots::Node * root = supervisor_->getRoot();
      std::vector<webots::Node*> childsOfRoot;
      //std::cerr<<"Get children"<<std::endl;
      WebotsExplorer::getChildren(root,1,childsOfRoot);
      //std::cerr<<"Good bad"<<std::endl;
      webots::Field * gravity(NULL);

      webots::Node* worldInfo = childsOfRoot[0];
      //std::cerr<<"get gravity pointer"<<std::endl;
      WebotsExplorer::getField(worldInfo,"gravity",&gravity);
      //std::cerr<<"value of pointer"<<gravity<<std::endl;

      accelerometer_=
          new Accelerometer(supervisor_->getAccelerometer("accelerometer"),
                            gravity,
                            Clock::timeStepInS());

	  robot_->registerForbiddenName("accelerometer");
      //    sensors_.assign(NUMBER_OF_SENSORS,NULL);
//
//    sensors_[LEFT_FORE_TOUCHSENSOR] = new Touch3DSensor(state_.GRFs[LEFT_FORE],
//                                                       0.5,0.0);
//    sensors_[RIGHT_FORE_TOUCHSENSOR] = new Touch3DSensor(state_.GRFs[RIGHT_FORE],
//                                                       0.5,0.0);
//    sensors_[LEFT_HIND_TOUCHSENSOR] = new Touch3DSensor(state_.GRFs[LEFT_HIND],
//                                                       0.5,0.0);
//    sensors_[RIGHT_HIND_TOUCHSENSOR] = new Touch3DSensor(state_.GRFs[RIGHT_HIND],
//                                                       0.5,0.0);
//
//    sensors_[LEFT_FORE_KNEE_CODER] =
//        new KneeCoder(supervisor_->getServo("LEFT_FORE_ANKLE_SERVO"),
//                      Clock::timeStepInMS());
//    sensors_[RIGHT_FORE_KNEE_CODER] =
//        new KneeCoder(supervisor_->getServo("RIGHT_FORE_ANKLE_SERVO"),
//                      Clock::timeStepInMS());
//    sensors_[LEFT_HIND_KNEE_CODER] =
//        new KneeCoder(supervisor_->getServo("LEFT_HIND_ANKLE_SERVO"),
//                      Clock::timeStepInMS());
//    sensors_[RIGHT_HIND_KNEE_CODER] =
//        new KneeCoder(supervisor_->getServo("RIGHT_HIND_ANKLE_SERVO"),
//                      Clock::timeStepInMS());
  }


  void ControllerPrimpleAmarsi::initModels(){
    state_.kinematicModels_.assign(4,(amarsi::LegKinematicModel*)0);

    state_.kinematicModels_[LEFT_FORE] =
        new LegKinematicModelAmarsi(supervisor_->getServo("LEFT_FORE_hip_servo"),
              supervisor_->getServo("LEFT_FORE_FRONT_KNEE_SERVO"),
              supervisor_->getServo("LEFT_FORE_ANKLE_SERVO"),
              supervisor_->getFromDef("LEFT_FORE_HIP_SERVO"),
              supervisor_->getFromDef("LEFT_FORE_FRONT_KNEE"),
              supervisor_->getFromDef("LEFT_FORE_ANKLE"),
              Clock::timeStepInMS());

    state_.kinematicModels_[RIGHT_FORE] =
           new LegKinematicModelAmarsi(supervisor_->getServo("RIGHT_FORE_hip_servo"),
                 supervisor_->getServo("RIGHT_FORE_FRONT_KNEE_SERVO"),
                 supervisor_->getServo("RIGHT_FORE_ANKLE_SERVO"),
                 supervisor_->getFromDef("RIGHT_FORE_HIP_SERVO"),
                 supervisor_->getFromDef("RIGHT_FORE_FRONT_KNEE"),
                 supervisor_->getFromDef("RIGHT_FORE_ANKLE"),
                 Clock::timeStepInMS());

    state_.kinematicModels_[LEFT_HIND] =
           new LegKinematicModelAmarsi(supervisor_->getServo("LEFT_HIND_hip_servo"),
                 supervisor_->getServo("LEFT_HIND_FRONT_KNEE_SERVO"),
                 supervisor_->getServo("LEFT_HIND_ANKLE_SERVO"),
                 supervisor_->getFromDef("LEFT_HIND_HIP_SERVO"),
                 supervisor_->getFromDef("LEFT_HIND_FRONT_KNEE"),
                 supervisor_->getFromDef("LEFT_HIND_ANKLE"),
                 Clock::timeStepInMS());

    state_.kinematicModels_[RIGHT_HIND] =
           new LegKinematicModelAmarsi(supervisor_->getServo("RIGHT_HIND_hip_servo"),
                 supervisor_->getServo("RIGHT_HIND_FRONT_KNEE_SERVO"),
                 supervisor_->getServo("RIGHT_HIND_ANKLE_SERVO"),
                 supervisor_->getFromDef("RIGHT_HIND_HIP_SERVO"),
                 supervisor_->getFromDef("RIGHT_HIND_FRONT_KNEE"),
                 supervisor_->getFromDef("RIGHT_HIND_ANKLE"),
                 Clock::timeStepInMS());

    robot_->registerForbiddenName("LEFT_FORE_FRONT_KNEE_SERVO");
    robot_->registerForbiddenName("LEFT_FORE_FRONT_KNEE_SERVO");
    robot_->registerForbiddenName("LEFT_FORE_FRONT_KNEE_SERVO");
    robot_->registerForbiddenName("RIGHT_FORE_hip_servo");
    robot_->registerForbiddenName("RIGHT_FORE_FRONT_KNEE_SERVO");
    robot_->registerForbiddenName("RIGHT_FORE_ANKLE_SERVO");
    robot_->registerForbiddenName("LEFT_HIND_FRONT_KNEE_SERVO");
    robot_->registerForbiddenName("LEFT_HIND_FRONT_KNEE_SERVO");
    robot_->registerForbiddenName("LEFT_HIND_FRONT_KNEE_SERVO");
    robot_->registerForbiddenName("RIGHT_HIND_hip_servo");
    robot_->registerForbiddenName("RIGHT_HIND_FRONT_KNEE_SERVO");
    robot_->registerForbiddenName("RIGHT_HIND_ANKLE_SERVO");

  }
}
