/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file MotionRestrictorManager.cpp
 *
 * \date 18 dec. 2009
 * \author Alexandre Tuleu
 */

#include "MotionRestrictorManager.h"
#include <sstream>
#include <iostream>
#include <algorithm>
#include <libamarsi-quad/common/CheetahMessages.h>
#include <libamarsi-quad/controller/Clock.h>


using namespace std;

namespace amarsi {
//==============================================================================
// DEAMBULATOR CONTROLLER
//==============================================================================


	bool MotionRestrictorManager::
		setDeambulatorStates(const std::string & configurationString){
		clear();
		istringstream iss(configurationString);
		string event;
		while(getline(iss,event,'|')){//for each group separated by a '|'
			istringstream isevent(event);
			string eventType;
			double time;
			isevent>>time;
			if(isevent.peek()==':'){
				isevent.ignore();
				isevent>>eventType;
			}else
				return false;
			if(eventType=="FREE"){
				registerEventNoSort(new MotionRestrictorManager::Event(time,SetMotionRestrictor::FREE));
			}else if(eventType=="SAGITAL_FIXED_PITCH"){
				registerEventNoSort(new MotionRestrictorManager::Event(time,SetMotionRestrictor::SAGITAL_FIXED_PITCH));
			}else if(eventType=="SAGITAL_FREE_PITCH"){
				registerEventNoSort(new MotionRestrictorManager::Event(time,SetMotionRestrictor::SAGITAL_FREE_PITCH));
			}else
				return false;
		}
		sort();
		return true;
	}


	MotionRestrictorManager::Event::Event(double triggerTime, int state)
	: amarsi::Event(triggerTime)
	, state_(state){

	}

	MotionRestrictorManager::Event::~Event(){

	}

	void MotionRestrictorManager::Event::action(){
		SetMotionRestrictor m(ID_ROBOT,ID_PHYSICS);
		m.setType(state_);
		m.send_me();
	}

	void MotionRestrictorManager::forceFreeAfter(double second){
		//get all event after seconds
		EventManager::iterator beg;
		bool smaller(true);
		for (;beg!=end() && smaller; ++beg){
			smaller = (*beg)->time() < second;
		}
		//erase all events after second


		for(EventManager::iterator it=beg;it!=end();++it){
			delete *it;
		}
		erase(beg,end());

		//now add a free event at second

		//this is the last, don't need to sort.
		registerEventNoSort(new MotionRestrictorManager::Event(second,
		                                                       SetMotionRestrictor::FREE));
	}
}
