/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file ChainOfFilter.h
 *
 * \date 25 nov. 2009
 * \author Alexandre Tuleu
 */

#ifndef CHAINOFFILTER_H_
#define CHAINOFFILTER_H_

#include "Filter.h"
#include <vector>
namespace Filtering {
  /**
   * Applies a sequence of Filter. Filter could be added or removed
   * using pushBackFilter() and removeFilter().
   * ChainOfFilter manage the life cycle of the given Filter.
   *
   * \todo try to implement the chain as an inherited class of ListOfFilter.
   * \todo Should copy the Filter rather than use pointer, or change to
   *       smart_pointer.
   * \todo Find a way to identify a particular filter,
   */
  class ChainOfFilter: public Filtering::Filter {
  protected :
    typedef std::vector<Filter*> ListOfFilter;
    /**
     * The ordered sequence of Filter to apply.
     */
    ListOfFilter filters_;

  public:
    /**
     * Standard constructor
     * @return an empty chain of Filter
     */
    ChainOfFilter();
    virtual ~ChainOfFilter();
    virtual double& output(double& input);

    /**
     * Add the filter at the end of the queue.
     * @param f a pointer to the filter to add Filter.
     */

    void pushBackFilter(Filter *f){
      filters_.push_back(f);
    }
    /**
     *
     * @param f
     */
    void removeFilter(Filter* f);



  };

}

#endif /* CHAINOFFILTER_H_ */
