/**
 * \file Discretizer.cpp
 *
 *  \date Jun 17, 2010
 *  \author tuleu
 */

#include "Discretizer.h"
#include <cmath>
namespace Filtering {

  Discretizer::Discretizer(double min__, double max__, unsigned int precision__)
    : min_(min__)
    , max_(max__)
    , precision_(precision__){
    init();
  }

  Discretizer::~Discretizer() {
  }

  double& Discretizer::output(double & input){
    input = std::floor((input-min_)*intervals_/distance_); //get discretized value
    input = input * distance_ / intervals_ + min_ ; //now return value to its domain
    return input;
  }

  void Discretizer::init(){
    if(min_>max_){
      double tmp=max_;
      max_=min_;
      min_=tmp;
      //should throw exception
    }
    if(min_==max_)
      max_+=1.0;//should throw exception

    if(precision_==0)
      precision_=1;
    distance_=max_-min_;
    intervals_=std::pow(2.0,(int)precision_);
  }
}
