/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file DoubleDerivative.h
 *
 * \date 10 déc. 2009
 * \author Alexandre Tuleu
 */

#ifndef DOUBLEDERIVATIVE_H_
#define DOUBLEDERIVATIVE_H_

#include "Filter.h"

namespace Filtering {
  /**
   * Computes the double derivation of the input signal. For this purpose the
   * following formula is employed :
   * \f[ y_{i}=\frac{x_{i}+x_{i-2}-2*x_{i-1}}{(\Delta t)^2} \f]
   *
   * \todo Implement also centered and left derivation formula, to improve
   * consistency of teh computations.
   *
   */
  class DoubleDerivative: public  Filtering::Filter {
    /*enum Mode{

    };*/
    /**
     * \f$ x_{i-1}\f$
     */
    double n1_;
    /**
     * \f$ x_{i-2} \f$
     */
    double n2_;
    /**
     * Indicates if n1_ and n2_ are initialized.
     *   -  if it equals to 0 : none
     *   -  if it equals to 1 : n1_ only.
     *   -  if it equals to 2 : both.
     */
    int initialized_;
    /**
     * \f$ (\Delta t)^2 \f$
     */
    double dt2_;

  public:
    /**
     * Standard Constructor
     * @param dt_ \f$ \Delta t \f$
     * @return
     */
    DoubleDerivative(double dt_);
    virtual ~DoubleDerivative();
    virtual double& output(double& input);
  };

}

#endif /* DOUBLEDERIVATIVE_H_ */
