/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Hysteresis.h
 *
 * \date 25 nov. 2009
 * \author Alexandre Tuleu
 */

#ifndef HYSTERESIS_H_
#define HYSTERESIS_H_

#include "Filter.h"

namespace Filtering {
/*
 * \f[y_i=\left \{
  \begin{aligned}
  1.0& if\; x_i>t_1\; and y_{i-1}=0\\
  0.0& if \; x_i<t_2\; and y_{i-1}=1\\
  y_{i-1}& else
  \end{aligned}
\right.\f]

 */

  /**
   * Hysteresis threshold. Has the following ouput :
   * <img src="../../images/hysteresis.png">
   */
  class Hysteresis: public Filtering::Filter {
  protected :
    /**
     * First (increasing) threshold \f$ t_1 \f$ .
     */
    double thres1_;
    /**
     * Second (decreasing) threshold \f$ t_2 \f$ .
     */
    double thres2_;

    bool state_;
  public:
    /**
     * Standard constructor.
     * @param thres1 \f$ t_1 \f$
     * @param thres2 \f$ t_2 \f$
     * @return
     */
    Hysteresis(double thres1, double thres2);
    virtual ~Hysteresis();
    virtual double& output(double& input);

    ///@name Getters and Setters
    //@{
    double getThresold1() const {
      return thres1_;
    }

    double getThresold2() const {
      return thres2_;
    }

    void setThresold1(double thres1) {
      this->thres1_ = thres1;
    }

    void setThresold2(double thres2) {
      this->thres2_ = thres2;
    }
    //@}
  };

}

#endif /* HYSTERESIS_H_ */
