/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file LowPass.h
 *
 * \date 25 nov. 2009
 * \author Alexandre Tuleu
 */

#ifndef LOWPASS_H_
#define LOWPASS_H_

#include "Filter.h"
#include <assert.h>
namespace Filtering {
  /**
   * 1-D Low Pass filter, using the following formula :
\f[
y_i=a*y_{i-1}+(1-a)*x_i
\f]
   * where \f$ 0\leq a < 1 \f$ is the amount of low-pass filtering.
   */
  class LowPass: public Filtering::Filter {
  private:
    /**
     * \f$ a \f$
     */
    double amount_;
    double previous_;
  public:
    /**
     * Standard Constructor
     * @param amount \f$ a \f$
     */
    LowPass(double amount);
    virtual ~LowPass();
    virtual double& output(double& input);
    /// @name Getters and Setters
    //@{
    double getAmount() const {
      return amount_;
    }

    void setAmount(double amount) {
      if(amount>=1.0)
        return;
      this->amount_ = amount;
    }
    //@}
  };

}

#endif /* LOWPASS_H_ */
