/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Threshold.h
 *
 * \date 25 nov. 2009
 * \author Alexandre Tuleu
 */

#ifndef THRESHOLD_H_
#define THRESHOLD_H_

#include "Filter.h"

namespace Filtering {
  /**
   * A threshold. Output is one if bigger than \f$ \theta \f$, 0 else.
   */
  class Threshold: public  Filtering::Filter {
  protected :
    /**
     * \f$ \theta \f$
     */
    double threshold_;
  public:
    /**
     * Standard constructor.
     * @param threshold \f$ \theta \f$
     * @return
     */
    Threshold(double threshold);
    virtual ~Threshold();
    virtual double& output(double& input);
    ///@name Getters and Setters
    //@{
    double getThreshold() const{
      return threshold_;
    }

    void setThreshold(double t){
      threshold_=t;
    }
    //@}
  };

}

#endif /* THRESHOLD_H_ */
