/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file FloorElement.cpp
 *
 *  \date Jun 28, 2010
 *  \author tuleu
 */

#include <iostream>

#include <webots/Field.hpp>
#include <webots/Node.hpp>

#include "FloorElement.h"

namespace amarsi {

  FloorElement::SetOfFloorElement FloorElement::s_elements;

  FloorElement::FloorElement(webots::Node * node, const State & active,
                             const FloorElement::State & inactive){
    addElement(node,active,inactive);
    s_elements.insert(this);
  }

  FloorElement::~FloorElement() {
    s_elements.erase(this);
  }


  void FloorElement::activate(){
    for(SetOfFloorElement::iterator element = s_elements.begin();
        element != s_elements.end();
        ++element){
      if(*element != this){
        (*element)->desactivateIntern();
      }
    }
    this->activateIntern();
  }

  void FloorElement::activateIntern(){
    for(SetOfElement::iterator ele=d_element.begin();
        ele != d_element.end(); ++ele)
      ele->activate();
  }

  void FloorElement::desactivateIntern(){
    for(SetOfElement::iterator ele=d_element.begin();
        ele != d_element.end(); ++ele)
      ele->desactivate();
  }

  FloorElement::State::State(const Vector3 & pos, const Vector<4> & ori)
  : position(pos)
  , orientation(ori){
  }

  FloorElement::State::~State(){
  }

  FloorElement::Element::Element(webots::Node *node,
                                 const FloorElement::State & active,
                                 const FloorElement::State & inactive)
  : d_translation(node->getField("translation"))
  , d_orientation(node->getField("rotation"))
  , d_node(node)
  , d_active(active)
  , d_inactive(inactive){
    if( !d_translation || !d_orientation){
      std::cerr<<"Did not found the translation mandatory field in structure, I quit."
               <<std::endl;
      exit(1);
    }
  }

  FloorElement::Element::~Element(){
  }

  void FloorElement::Element::activate(){
    setState(d_active);

  }

  void FloorElement::Element::desactivate(){
    setState(d_inactive);
  }

  void FloorElement::Element::setState(FloorElement::State &s){
    d_translation->setSFVec3f(s.position.toArray());
    if(d_orientation)
      d_orientation->setSFRotation(s.orientation.toArray());
  }
}
