/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file FloorElement.h
 *
 *  \date Jun 28, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_QUAD_FLOORELEMENT_H_
#define LIBAMARSI_QUAD_FLOORELEMENT_H_

#include <set>
#include <vector>
#include <libamarsi-quad/basic-math/Vector3.h>

namespace webots{
  class Field;
  class Node;
}


namespace amarsi {
  /**
   *
   */
  class FloorElement {
  public:
    virtual ~FloorElement();

    void activate();

  protected:

    struct State{
      Vector3 position;
      Vector<4> orientation;
      State(const Vector3 &, const Vector<4> &);
      virtual ~State();
    };

    FloorElement(webots::Node * node,
                 const State & active,
                 const State & inactive);

    State & activeState(unsigned int i=0);
    void addElement(webots::Node * n,
                    const State & active,
                    const State & inactive);
  private:
    class Element {
      friend class FloorElement;
    public:
      Element(webots::Node *node, const State & active, const State & inactive);
      virtual ~Element();
      void activate();
      void desactivate();
    private:
      void setState(State &s);
      webots::Field *d_translation;
      webots::Field *d_orientation;
      webots::Node * d_node;
      State d_active;
      State d_inactive;
    };
    void activateIntern();
    void desactivateIntern();


    typedef std::set<FloorElement*> SetOfFloorElement;
    typedef std::vector<Element> SetOfElement;

    static SetOfFloorElement s_elements;

    SetOfElement d_element;
  };

  inline FloorElement::State& FloorElement::activeState(unsigned int i){
    if(i<d_element.size())
      return d_element[i].d_active;
    /// \todo should throw exception here
    return d_element[0].d_active;
  }

  inline void FloorElement::addElement(webots::Node *n,
                                const FloorElement::State & active,
                                const FloorElement::State & inactive){
    d_element.push_back(Element(n,active,inactive));
  }

}
#endif // LIBAMARSI_QUAD_FLOORELEMENT_H_
