/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file LegKinematicModel.h
 *
 *  \date Jul 14, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_QUAD_LEGKINEMATICMODEL_AMARSI_H_
#define LIBAMARSI_QUAD_LEGKINEMATICMODEL_AMARSI_H_


#include "LegKinematicModel.h"


namespace webots{
  class Servo;
  class Node;
  class Field;
}


namespace amarsi {

  class LegKinematicModelAmarsi : public LegKinematicModel{
  public:
    LegKinematicModelAmarsi(webots::Servo* hip,
                      webots::Servo* knee1,
                      webots::Servo * knee2,
                      webots::Node * hipNode,
                      webots::Node * knee1Node,
                      webots::Node * knee2Node,
                      unsigned int timeStep);
    virtual ~LegKinematicModelAmarsi();

    /**
     * Computes the forward kinematic model for one given position.
     * The intended hipAngle are the one given by the Hips Actuator .
     * The intended kneeAngle are the one we got back from the PositionEncoder.
     * Note that if your provide other angle it will end up with wrong values
     * @param hipAngle
     * @param kneeAngle
     * @param positionInHipFrame
     * @param positionInBodyFrameAthipJoint
     * @param positionInBodyFrame
     */
    void forwardKinematic(double hipAngle, double kneeAngle,
                          Vector<2> & positionInHipFrame,
                          Vector<2> & positionInBodyFrameAthipJoint,
                          Vector3 & positionInBodyFrame) const ;
//    /**
//     * Computes the inverse kinematic model for one joint. If it fails to found
//     * a configuration, it will return false.
//     * @param positionInBodyFrame the expected end effector position
//     * @param hipAngle the corresponding order for the hip Actuator .
//     * @param kneeAngle the corresponding order for the knee Actuator
//     * @return \c true if succesfully found a corresponding configuration.
//     */
//    bool inverseKinematics(const Vector3 & positionInBodyFrame,
//                           double & hipAngle,
//                           double & kneeAngle);
    /**
     * Computes the inverse kinematic model for one joint. If it fails to found
     * a configuration, it will return false.
     *
     *
     *
     * @param positionInBodyFrame the expected end effector position
     * @param hipAngle the corresponding order for the hip Actuator .
     * @param kneeAngle the corresponding order for the knee Actuator
     * @return \c true if succesfully found a corresponding configuration.
    */
    bool inverseKinematics(const Vector<2> & positionInBodyFrameAtHipJoint,
                           double & hipAngle,
                           double & kneeAngle) const;

  protected :
    virtual void update();
  private :


    void init(webots::Node * hipNode, webots::Node * knee1Node,
              webots::Node * knee2Node, unsigned int timeStep);
    void clear();
    void updateEquivalent();

    webots::Field * getField(webots::Node* n, const std::string & name);

    /**
       * Computes the forward kinematic model of the leg. Please note that there
       * is two knee angle instead of one. Indeed the second servo of the knee
       * joint may have some delay in pursuiving the position of the compliant
       * one.
       *
       * \internal
       *
       * @param hipAngle
       * @param knee1Angle
       * @param knee2Angle
       * @param positionInHipFrame
       * @param positionInBodyFrameAthipJoint
       * @param positionInBodyFrame
       */
      void forwardKinematic(double hipAngle, double knee1Angle, double knee2Angle,
                            Vector<2> & positionInHipFrame,
                            Vector<2> & positionInBodyFrameAthipJoint,
                            Vector3 & positionInBodyFrame) const;



    //model information
    double d_alpha0TotalOffset;
    //model offset and servo offset (\alpha_{o_1} + \pi )
    double d_alpha1TotalOffset;
    //model offste and servo ofset ( \frac{pi}{2} \alpha_{o_2} )
    double d_alpha2TotalOffset;

    double d_kneeMaxPosition;
    double d_kneeMinPosition;
    double d_hipMinPosition;
    double d_hipMaxPosition;

    double d_segmentLength;

    Vector3 d_hipPositionInBodyFrame;

    webots::Servo* d_hip;
    webots::Servo* d_knee1;
    webots::Servo* d_knee2;
  };

}

#endif // LIBAMARSI_QUAD_LEGKINEMATICMODEL_H_
