/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Accelerometer.cpp
 *
 *  \date Jun 18, 2010
 *  \author tuleu
 */

#include <webots/Accelerometer.hpp>
#include <webots/Field.hpp>
#include "Accelerometer.h"

namespace amarsi {

  Accelerometer::Accelerometer(webots::Accelerometer * accelerometer,
                               webots::Field* field,
                               double timestep)
  : acc_(accelerometer)
  , timestep_(timestep)
  , correction_(field->getSFVec3f()){
    init();

  }

  Accelerometer::~Accelerometer() {
  }


  void Accelerometer::init(){
    acc_->enable((int)(timestep_*1000.0));
  }

  void Accelerometer::update(){
    //get the value from webots
    acceleration_=Vector3(acc_->getValues())+correction_;//we have to remove gravity !
    //just integrate twice.
    speed_+=timestep_*acceleration_;
    relativePosition_+=timestep_*speed_;
  }
}


std::ostream & operator<<(std::ostream & out, const amarsi::Accelerometer & acc){
  return out<<"Accelerometer : [ acceleration : "<< acc.acceleration()
                           <<" , speed :"<< acc.speed()
                           <<" , displacement : "<< acc.displacement() <<" ]";
}
