/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Gyro.cpp
 *
 *  \date Jun 18, 2010
 *  \author tuleu
 */

#include <webots/Gyro.hpp>

#include "Gyro.h"

namespace amarsi {

  Gyro::Gyro(webots::Gyro * gyro, double timestep)
  : gyro_(gyro)
  , timestep_(timestep){
    init();
  }

  Gyro::~Gyro() {
  }

  void Gyro::init(){
    gyro_->enable((int)(timestep_*1000.0));
  }

  void Gyro::update(){
    //gets the value
    speed_=Vector3(gyro_->getValues());
    //integrate
    orientation_+=timestep_*speed_;
  }
}

std::ostream & operator<<(std::ostream & out, const amarsi::Gyro & gyro){
  return out<<"Gyro : [ speed : "<< gyro.speed()
            <<" , orientation : "<< gyro.orientation() <<"]";
}
