/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Gyro.h
 *
 *  \date Jun 18, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_QUAD_GYRO_H_
#define LIBAAMARSI_QUAD_GYRO_H_

#include <libamarsi-quad/basic-math/Vector3.h>

#include "Sensor.ih"

namespace webots {
  class Gyro;
}

namespace amarsi {
  /**
   * Represents a Gyroscope that computes relative orientation using a simple
   * euler integration.
   *
   * This class is all purpose class, if you want the specifications of the
   * simplified Cheetah model please refers to \ref sensors and \ref model .
   * \ingroup sensors
   * \ingroup user
   */
  class Gyro : public amarsi::Sensor {
  public:
    ///@name Constructors and Destructor
    //@{
    Gyro(webots::Gyro * gyro, double timestep_);
    virtual ~Gyro();
    //@}

    ///@name accessors
    //{
    /**
     * Angular speed of the robot during the simulation step.
     * @return a reference to the angular speed during the current simulation
     *         step.
     */
    const Vector3 & speed() const;
    /**
     * Current orientation of the robot during the simulation step. This is
     * reseted by reset(), and each time the simulation is reseted by
     * Supervisor::simulationReset()
     * @return a reference to the angular speed during the current simulation
     *         step.
     */
    const Vector3 & orientation() const;

    //}
    /**
     * Resets all the integrated orientation();
     */
    void reset();

  protected :
    virtual void update();
  private :
    void init();
    webots::Gyro * gyro_;
    double timestep_;
    Vector3 orientation_,speed_;
  };

//==============================================================================
// INLINE IMPLEMENTATION
//==============================================================================

  inline const Vector3 & Gyro::orientation() const{
    return orientation_;
  }

  inline const Vector3 & Gyro::speed() const{
   return speed_;
  }

  inline void Gyro::reset(){
    orientation_=Vector3::ZERO;
  }

}

std::ostream & operator<<(std::ostream & out, const amarsi::Gyro & gyro);

#endif // LIBAMARSI_QUAD_GYRO_H_
