/*
 * Copyright (c) 2010 Ecole Polytechnique Fédérale de Lausanne
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file PositionEncoder.h
 *
 *  \date May 18, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_QUAD_POSITION_ENCODER_H_
#define LIBAMARSI_QUAD_POSITION_ENCODER_H_
#include <iostream>
#include "ServoPositionSensor.h"
#include <libamarsi-quad/filtering/filtering.h>
namespace amarsi {
  /**
   * Represent a rotation encoder placed on a fixed joint.
   *
   * Even if the encoder output radian values, it has a limited precison.
   * Values are returned between min() and max(), and have a precision on
   * precison() bits.
   *
   * This class is all purpose class, if you want the specifications of the
   * simplified Cheetah model please refers to \ref sensors and \ref model .
   * \ingroup sensors
   *
   * \ingroup user
   */
  class PositionEncoder : public ServoPositionSensor {
  public:
    PositionEncoder(webots::Servo* s, int timestep,
                    double min, double max, unsigned int precision,bool reverse);
    virtual ~PositionEncoder();

  protected :
    virtual void update();
  private :
    Filtering::Discretizer discretizer;
  };

}
std::ostream & operator<<(std::ostream & out,
                          const amarsi::PositionEncoder & enc);
#endif // LIBAMARSI_QUAD_POSITION_ENCODER_H_
