/*
 * Copyright (c) 2010 Ecole Polytechnique Fédérale de Lausanne
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file ServoPositionSensor.h
 *
 *  \date May 18, 2010
 *  \author tuleu
 */

#ifndef LIBAMARSI_QUAD_SERVOPOSITIONSENSOR_H_
#define LIBAMARSI_QUAD_SERVOPOSITIONSENSOR_H_

#include "Sensor.ih"

namespace webots{
  class Servo;
}

namespace amarsi {

  /**
   * A Sensor that simply take a position value from a webots servo.
   *
   * \ingroup internal
   */
  class ServoPositionSensor : public Sensor{
  public:
    /**
     * Standard constructor.
     * @param s the webots::Servo to use.
     * @param timestep the timestep of the simulation in ms.
     */
    ServoPositionSensor(webots::Servo *s, int timestep, bool reverse);
    virtual ~ServoPositionSensor();

    /**
     * The position in radians.
     * @return the position in radians.
     */
    double position() const;

  protected :
    virtual void update();

    double position_;
  private :
    bool reverse_;
    const webots::Servo *s_;

  };
  inline double ServoPositionSensor::position() const {
    return position_;
  }
}

#endif // libamarsi-quad_SERVOPOSITIONSENSOR_H_
