/*
 * Copyright (c) 2009-2010 Alexandre Tuleu
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Touch3DSensor.cpp
 *
 * \date 18 déc. 2009
 * \author Alexandre Tuleu
 */

#include "Touch3DSensor.h"

namespace amarsi {

  Touch3DSensor::Touch3DSensor(const Vector3& value, double lpAmount,
                               double threshold, double min, double max,
                               unsigned int precision)
    : value_(value)
    , discretizedValue_(value)
    , contact_(false)
    , lp_(lpAmount)
    , th_(threshold)
    , d_(min,max,precision){
  }

  Touch3DSensor::~Touch3DSensor() {
  }

  void Touch3DSensor::update(){
    discretizedValue_=value_;
    //perform the lowpass lp then the thresholding
    contact_=(0.0 != value_.norm()>>lp_>>th_);
    d_.output(discretizedValue_[0]);
    d_.output(discretizedValue_[1]);
    d_.output(discretizedValue_[2]);
  }
}

std::ostream & operator<<(std::ostream & out,
                          const amarsi::Touch3DSensor & touch){
  return out<<"Touch3DSensor : [ non-encoded value : "<<touch.value_
                           <<" , returned value : "<<touch.discretizedValue_
                           <<" , is in contact : "<<(touch.contact_?"yes":"no")
                           <<" ] ";
}
