/*
 * Copyright (c) 2010 EPFL (Ecole Polytechnique Fédérale de Lausanne)
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Lock.cpp
 *
 *  \date Aug 26, 2010
 *  \author tuleu
 */

#include "Lock.h"
#include "Debug.h"
namespace amarsi {

  Lock::Lock()
  : d_locked(false){
  }

  Lock::~Lock(){
  }

  bool Lock::operator ()(){
    return d_locked;
  }

  Lock::Key Lock::getAKey(){
    if(d_locked)
      return Lock::Key((Lock*)0);
    return Lock::Key(this);
  }

  Lock::Key::Key(Lock *associatedLock)
    : d_associatedLock(associatedLock){
  }

  Lock::Key::~Key(){

  }

  bool Lock::Key::isAValidKey(){
    return d_associatedLock != NULL;
  }

  void Lock::Key::lock(){
    if(isAValidKey()){
      d_associatedLock->d_locked = true;
      debug_supervisor("Key "<<this<<" locked lock "<<d_associatedLock<<"."<<std::endl);
    }
  }

  void Lock::Key::unlock(){
    if(isAValidKey()){
      d_associatedLock->d_locked = false;
      debug_supervisor("Key "<<this<<" unlocked lock "<<d_associatedLock<<"."<<std::endl);
    }
  }

  const Lock::Key & Lock::Key::getAnInvalidKey(){
    return s_invalidKey;
  }

  Lock::Key Lock::Key::s_invalidKey((Lock*)0);
}
