/*
 * Copyright (c) 2010 Ecole Polytechnique Fédérale de Lausanne
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file Pref.h
 *
 *  \date May 11, 2010
 *  \author Alexandre Tuleu
 */

#ifndef libamarsi_PREF_H_
#define libamarsi_PREF_H_

#include "PreferenceReader.h"

template <typename T>
class Pref {
public:
	Pref(const std::string & domain, const std::string & name);
	virtual ~Pref();

	const T& value();
	const T& operator() ();
private :
	const std::string domain_;
	const std::string name_;

	void update();

	T value_;
	unsigned int lastNumberOfParsedFiles_;
};


template <typename T>
	const T& Pref<T>::value(){
	if(PreferenceReader::instance().nbParsedFiles>lastNumberOfParsedFiles_){
		update();
	}
	return value_;
}


template <typename T>
	void Pref<T>::update(){
	PreferenceReader::instance().getValue(domain_,name_,value_);
	lastNumberOfParsedFiles_=PreferenceReader::instance().nbParsedFiles();
}

template <typename T>
	const T& Pref<T>::operator ()(){
	return value();
}

template <typename T>
	Pref<T>::Pref(const std::string & domain, const std::string & name)
	 : domain_(domain)
	 , name_(name)
	 , lastNumberOfParsedFiles_(0){
}


template <typename T>
	virtual ~Pref(){
}

#endif // libamarsi_PREF_H_
