/*
 * Copyright (c) 2010 Ecole Polytechnique Fédérale de Lausanne
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "PreferenceReader.h"
#include <iostream>
#include <sstream>

PreferenceReader::PreferenceReader()
  : stream_(NULL)
  , currentDomain_(domains_.end()){
  init();
}
/*PreferenceReader::PreferenceReader(const std::string & file)
  : file_(new std::ifstream(file.c_str()))
  , currentDomain_(NULL){
  init();
  parse();
}*/

void PreferenceReader::init(){
  domains_["void"]=Domain();
  currentDomain_=domains_.find("void");
}

PreferenceReader::~PreferenceReader(){
  clear();
}
void PreferenceReader::read(const std::string & file){
  std::ifstream file_(file.c_str());
  read(file_);
}
void PreferenceReader::read(std::istream & stream){
  stream_ = & stream;
  parse();
  stream_ = NULL;
}

void PreferenceReader::clear(){
  if(stream_){
    delete stream_;
  }
  domains_.clear();
}

PreferenceReader::Domain PreferenceReader::s_voidDomain;

const PreferenceReader::Domain & PreferenceReader::currentDomainValues() const {
	if(currentDomain_ == domains_.end()){
		return s_voidDomain;
	}
	return currentDomain_->second;
}

bool PreferenceReader::setDomain(const std::string & name){
  ListOfDomain::const_iterator it = domains_.find(name);
  if(it==domains_.end())
    return false;
  currentDomain_=it;
  return true;
}


void PreferenceReader::parse(){
  if( ! stream_ )
    return;
  std::string line;
  while(getline(*stream_,line)){//for each line of the file
    std::istringstream lss(line);
    lss>>std::skipws;
    //first skip any ws
    lss>>std::ws;
    if(lss.bad())//end line, we have a blank line
      continue;//stop loop

    if(lss.peek()=='#')//it's a comment
    	continue;
    if(lss.peek()=='['){//specifying a domain
      std::string name;
      lss.ignore();
      getline(lss,name,']');
      if(!setDomain(name)){
      	currentDomain_=(domains_.insert(std::make_pair(name,Domain()))).first;
      }
    }else{//specifying a pref;
      std::ostringstream name;//use a oss, since we have too much specific case to handle
      while(lss.good()&& lss.peek()!=' ' && lss.peek()!='=')//get the name
      	name<<(((char)lss.get()));
      lss>>std::ws;
      if(lss.peek()!='=')//parsing error \todo should throw an exception
      	continue;
      lss.ignore();
      lss>>std::ws;
      lss>>std::noskipws;
      std::string value;

      getline(lss,value,'#');//skip endline comment
      //removing ending blank characther

      std::string::reverse_iterator it =value.rbegin();

      while(it!=value.rend()&& *it==' '){
      	++it;
      }
      if(it.base()!=value.end())
      	value.erase(it.base(),value.end());
      domains_[currentDomain_->first][name.str()]=value;//insert data.
    }
  }
}

PreferenceReader & PreferenceReader::instance(){
  return r;
}

PreferenceReader PreferenceReader::r;


std::ostream & operator<<(std::ostream &out, const PreferenceReader &r){

  std::string domainName;
  for (PreferenceReader::ListOfDomain::const_iterator domain=r.domains_.begin();
       domain!=r.domains_.end();++domain){
    domainName=domain->first;
    for(PreferenceReader::Domain::const_iterator value=domain->second.begin();
        value != domain->second.end();++value){
      out<<domainName<<"."<<value->first<<" = "<<value->second<<std::endl;
    }
  }
  return out;
}
