/*
 * Copyright (c) 2010 Ecole Polytechnique Fédérale de Lausanne
 *
 * This file is part of libamarsi-quad.
 *
 * libamarsi-quad is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libamarsi-quad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libamarsi-quad.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 * \file PreferenceReaderTest.cpp
 *
 *  \date May 10, 2010
 *  \author Alexandre Tuleu
 */

#include "PreferenceReaderTest.h"


TEST_F(PreferenceReaderTest,DomainAreDetected){
	EXPECT_TRUE(PreferenceReader::instance().setDomain("core"));
	EXPECT_TRUE(PreferenceReader::instance().setDomain("component"));
	EXPECT_FALSE(PreferenceReader::instance().setDomain("dooesnotexist"));
}

TEST_F(PreferenceReaderTest, OptionAreDetected){
	EXPECT_TRUE(PreferenceReader::instance().setDomain("core"));
	int tmpInt;
	EXPECT_TRUE(PreferenceReader::instance().getValue("truc1",tmpInt));
	EXPECT_EQ(1,tmpInt);
	float tmpFloat;
	EXPECT_TRUE(PreferenceReader::instance().getValue("truc2",tmpFloat));
	EXPECT_FLOAT_EQ(2.35,tmpFloat);
	std::string str;
	EXPECT_TRUE(PreferenceReader::instance().getValue("string",str));
	EXPECT_EQ("test string",str);
	bool opt;
	EXPECT_TRUE(PreferenceReader::instance().getValue("component","option",opt));
}

TEST_F(PreferenceReaderTest, CommentAreWorking){
	//some of the following option are redefined or defined in comment,
	//so they should not exist/change
	bool opt;
	EXPECT_TRUE(PreferenceReader::instance().getValue("component","option",opt));
	EXPECT_EQ(true,opt);
	EXPECT_FALSE(PreferenceReader::instance().setDomain("cached"));
}

TEST_F(PreferenceReaderTest, MaskingIsWorking){
	float tmpFloat;
	EXPECT_TRUE(PreferenceReader::instance().getValue("core","truc2",tmpFloat));
	EXPECT_FLOAT_EQ(2.35,tmpFloat);
	PreferenceReader::instance().read("/tmp/test-utils2.pref");
	EXPECT_TRUE(PreferenceReader::instance().getValue("core","truc3",tmpFloat));
	EXPECT_FLOAT_EQ(3.36,tmpFloat);
	EXPECT_TRUE(PreferenceReader::instance().getValue("core","truc2",tmpFloat));
	EXPECT_FLOAT_EQ(0,tmpFloat);
}

TEST_F(PreferenceReaderTest , BooleanParseIsWorking){
  bool val1,val2,val3,val4,val5,val6,val7;
  EXPECT_TRUE(PreferenceReader::instance().getValue("core2","val1",val1));// "true"
  EXPECT_TRUE(PreferenceReader::instance().getValue("core2","val2",val2));// "1"
  EXPECT_TRUE(PreferenceReader::instance().getValue("core2","val3",val3));// "0"
  EXPECT_TRUE(PreferenceReader::instance().getValue("core2","val4",val4));// "false"
  EXPECT_TRUE(PreferenceReader::instance().getValue("core2","val5",val5));// ""
  EXPECT_TRUE(PreferenceReader::instance().getValue("core2","val6",val6));//"no"
  EXPECT_TRUE(PreferenceReader::instance().getValue("core2","val7",val7));//"disabled"
  EXPECT_EQ(true,val1);
  EXPECT_EQ(true,val2);
  EXPECT_EQ(true,val3);
  EXPECT_EQ(false,val4);
  EXPECT_EQ(false,val5);
  EXPECT_EQ(false,val6);
  EXPECT_EQ(false,val7);
}
